@@ -52,14 +52,11 @@
 # include <ncurses/ncurses.h>
 #endif
 
-#ifdef HAVE_WIDECHAR
-# include <wctype.h>
-# include <wchar.h>
-#endif
-
 #include "c.h"
+#include "cctype.h"
 #include "closestream.h"
 #include "nls.h"
+#include "widechar.h"
 #include "strutils.h"
 #include "xalloc.h"
 #include "mbsalign.h"
@@ -69,7 +66,7 @@
 #include "list.h"
 #include "blkdev.h"
 
-static const char *default_disks[] = {
+static const char *const default_disks[] = {
 #ifdef __GNU__
 		"/dev/hd0",
 		"/dev/sd0",
@@ -211,14 +208,6 @@ static struct cfdisk_menuitem main_menuitems[] = {
 	{ 0, NULL, NULL }
 };
 
-/* extra partinfo in name:value pairs */
-struct cfdisk_extra {
-	char *name;
-	char *data;
-
-	struct list_head exs;
-};
-
 /* line and extra partinfo list_head */
 struct cfdisk_line {
 	char			*data;		/* line data */
@@ -255,10 +244,10 @@ struct cfdisk {
 	struct libmnt_table *fstab;
 	struct libmnt_cache *mntcache;
 #endif
-	unsigned int	wrong_order :1,		/* PT not in right order */
-			zero_start :1,		/* ignore existing partition table */
-			device_is_used : 1,	/* don't use re-read ioctl */
-			show_extra :1;		/* show extra partinfo */
+	bool	wrong_order,	/* PT not in right order */
+		zero_start,	/* ignore existing partition table */
+		device_is_used,	/* don't use re-read ioctl */
+		show_extra;	/* show extra partinfo */
 };
 
 
@@ -958,6 +947,7 @@ static void menu_set_title(struct cfdisk_menu *m, const char *title)
 			m->width = len + MENU_TITLE_PADDING;
 		str = xstrdup(title);
 	}
+	free(m->title);
 	m->title = str;
 }
 
@@ -1965,7 +1955,7 @@ static int ui_get_size(struct cfdisk *cf,	/* context */
 				insec = 1;
 				buf[len - 1] = '\0';
 			}
-			rc = parse_size(buf, (uintmax_t *)&user, &pwr);	/* parse */
+			rc = ul_parse_size(buf, (uintmax_t *)&user, &pwr);	/* parse */
 		}
 
 		if (rc == 0) {
@@ -2184,7 +2174,8 @@ static int ui_create_label(struct cfdisk *cf)
 	nitems = fdisk_get_nlabels(cf->cxt);
 	cm = xcalloc(nitems + 1, sizeof(struct cfdisk_menuitem));
 
-	while (fdisk_next_label(cf->cxt, &lb) == 0) {
+	while (fdisk_next_label(cf->cxt, &lb) == 0 && i < nitems) {
+
 		if (fdisk_label_is_disabled(lb) ||
 		    fdisk_label_get_type(lb) == FDISK_DISKLABEL_BSD)
 			continue;
@@ -2255,7 +2246,7 @@ done:
 static int ui_help(void)
 {
 	size_t i;
-	static const char *help[] = {
+	static const char *const help[] = {
 		N_("This is cfdisk, a curses-based disk partitioning program."),
 		N_("It lets you create, delete, and modify partitions on a block device."),
 		"  ",
@@ -2519,7 +2510,7 @@ static int main_menu_action(struct cfdisk *cf, int key)
 			  buf, sizeof(buf));
 
 		ref = 1;
-		if (rc <= 0 || (strcasecmp(buf, "yes") != 0 &&
+		if (rc <= 0 || (c_strcasecmp(buf, "yes") != 0 &&
 				strcasecmp(buf, _("yes")) != 0)) {
 			info = _("Did not write partition table to disk.");
 			break;
@@ -2739,6 +2730,8 @@ static void __attribute__((__noreturn__)) usage(void)
 	      _("     --lock[=<mode>]      use exclusive device lock (%s, %s or %s)\n"), "yes", "no", "nonblock");
 	fputs(_(" -r, --read-only          forced open cfdisk in read-only mode\n"), out);
 
+	fputs(_(" -b, --sector-size <size> physical and logical sector size\n"), out);
+
 	fputs(USAGE_SEPARATOR, out);
 	fprintf(out, USAGE_HELP_OPTIONS(26));
 
@@ -2751,6 +2744,7 @@ int main(int argc, char *argv[])
 	const char *diskpath = NULL, *lockmode = NULL;
 	int rc, c, colormode = UL_COLORMODE_UNDEF;
 	int read_only = 0;
+	size_t user_ss = 0;
 	struct cfdisk _cf = { .lines_idx = 0 },
 		      *cf = &_cf;
 	enum {
@@ -2760,6 +2754,7 @@ int main(int argc, char *argv[])
 		{ "color",   optional_argument, NULL, 'L' },
 		{ "lock",    optional_argument, NULL, OPT_LOCK },
 		{ "help",    no_argument,       NULL, 'h' },
+		{ "sector-size", required_argument, NULL, 'b' },
 		{ "version", no_argument,       NULL, 'V' },
 		{ "zero",    no_argument,	NULL, 'z' },
 		{ "read-only", no_argument,     NULL, 'r' },
@@ -2771,8 +2766,15 @@ int main(int argc, char *argv[])
 	textdomain(PACKAGE);
 	close_stdout_atexit();
 
-	while((c = getopt_long(argc, argv, "L::hVzr", longopts, NULL)) != -1) {
+	while((c = getopt_long(argc, argv, "b:L::hVzr", longopts, NULL)) != -1) {
 		switch(c) {
+		case 'b':
+			user_ss = strtou32_or_err(optarg,
+					_("invalid sector size argument"));
+			if (user_ss != 512 && user_ss != 1024 &&
+			    user_ss != 2048 && user_ss != 4096)
+				errx(EXIT_FAILURE, _("invalid sector size argument"));
+			break;
 		case 'h':
 			usage();
 			break;
@@ -2811,6 +2813,8 @@ int main(int argc, char *argv[])
 	cf->cxt = fdisk_new_context();
 	if (!cf->cxt)
 		err(EXIT_FAILURE, _("failed to allocate libfdisk context"));
+	if (user_ss)
+		fdisk_save_user_sector_size(cf->cxt, user_ss, user_ss);
 
 	fdisk_set_ask(cf->cxt, ask_callback, (void *) cf);
 
