@@ -128,27 +128,28 @@ struct entry {
 static void __attribute__((__noreturn__)) usage(void)
 {
 	fputs(USAGE_HEADER, stdout);
-	printf(_(" %s [-h] [-v] [-b blksize] [-e edition] [-N endian] [-i file] [-n name] dirname outfile\n"),
+	fprintf(stdout, _(" %s [-h] [-v] [-b blksize] [-e edition] [-N endian] [-i file] [-n name] dirname outfile\n"),
 		program_invocation_short_name);
 	fputs(USAGE_SEPARATOR, stdout);
-	puts(_("Make compressed ROM file system."));
+	fputsln(_("Make compressed ROM file system."), stdout);
 	fputs(USAGE_OPTIONS, stdout);
-	puts(_(  " -v             be verbose"));
-	puts(_(  " -E             make all warnings errors (non-zero exit status)"));
-	puts(_(  " -b blksize     use this blocksize, must equal page size"));
-	puts(_(  " -e edition     set edition number (part of fsid)"));
-	printf(_(" -N endian      set cramfs endianness (%s|%s|%s), default %s\n"), "big", "little", "host", "host");
-	puts(_(  " -i file        insert a file image into the filesystem"));
-	puts(_(  " -n name        set name of cramfs filesystem"));
-	printf(_(" -p             pad by %d bytes for boot code\n"), PAD_SIZE);
-	puts(_(  " -s             sort directory entries (old option, ignored)"));
-	puts(_(  " -z             make explicit holes"));
-	puts(_(  " -l[=<mode>]    use exclusive device lock (yes, no or nonblock)"));
-	puts(_(  " dirname        root of the filesystem to be compressed"));
-	puts(_(  " outfile        output file"));
+	fputsln(_(  " -v             be verbose"), stdout);
+	fputsln(_(  " -E             make all warnings errors (non-zero exit status)"), stdout);
+	fputsln(_(  " -b blksize     use this blocksize, must equal page size"), stdout);
+	fputsln(_(  " -e edition     set edition number (part of fsid)"), stdout);
+	fprintf(stdout, _(" -N endian      set cramfs endianness (%s|%s|%s), default %s\n"), "big", "little", "host", "host");
+	fputsln(_(  " -i file        insert a file image into the filesystem"), stdout);
+	fputsln(_(  " -n name        set name of cramfs filesystem"), stdout);
+	fprintf(stdout, _(" -p             pad by %d bytes for boot code\n"), PAD_SIZE);
+	fputsln(_(  " -s             sort directory entries (old option, ignored)"), stdout);
+	fputsln(_(  " -z             make explicit holes"), stdout);
+	fputsln(_(  " -l[=<mode>]    use exclusive device lock (yes, no or nonblock)"), stdout);
 	fputs(USAGE_SEPARATOR, stdout);
-	printf(USAGE_HELP_OPTIONS(16));
-	printf(USAGE_MAN_TAIL("mkfs.cramfs(8)"));
+	fputsln(_(  " dirname        root of the filesystem to be compressed"), stdout);
+	fputsln(_(  " outfile        output file"), stdout);
+	fputs(USAGE_SEPARATOR, stdout);
+	fprintf(stdout, USAGE_HELP_OPTIONS(16));
+	fprintf(stdout, USAGE_MAN_TAIL("mkfs.cramfs(8)"));
 	exit(MKFS_EX_OK);
 }
 
@@ -239,14 +240,6 @@ identical_file(struct entry *e1, struct entry *e2){
 	return equal;
 }
 
-/*
- * The longest file name component to allow for in the input directory tree.
- * Ext2fs (and many others) allow up to 255 bytes.  A couple of filesystems
- * allow longer (e.g. smbfs 1024), but there isn't much use in supporting
- * >255-byte names in the input directory tree given that such names get
- * truncated to 255 bytes when written to cramfs.
- */
-#define MAX_INPUT_NAMELEN 255
 
 static int find_identical_file(struct entry *orig, struct entry *new, loff_t *fslen_ub)
 {
@@ -293,7 +286,7 @@ static int cramsort (const struct dirent **a, const struct dirent **b)
 
 static unsigned int parse_directory(struct entry *root_entry, const char *name, struct entry **prev, loff_t *fslen_ub)
 {
-	struct dirent **dirlist;
+	struct dirent **dirlist = NULL;
 	int totalsize = 0, dircount, dirindex;
 	char *path, *endpath;
 	size_t len = strlen(name);
@@ -497,7 +490,7 @@ static unsigned int write_directory_structure(struct entry *entry, char *base, u
 			if (entry->child) {
 				if (stack_entries >= stack_size) {
 					stack_size *= 2;
-					entry_stack = xrealloc(entry_stack, stack_size * sizeof(struct entry *));
+					entry_stack = xreallocarray(entry_stack, stack_size, sizeof(struct entry *));
 				}
 				entry_stack[stack_entries] = entry;
 				stack_entries++;
