@@ -58,6 +58,7 @@
  *
  */
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <time.h>
 #include <unistd.h>
@@ -117,8 +118,7 @@ struct fs_control {
 	size_t fs_dirsize;		/* maximum size of directory */
 	unsigned long fs_inodes;	/* number of inodes */
 	int fs_magic;			/* file system magic number */
-	unsigned int
-	 check_blocks:1;		/* check for bad blocks */
+	bool check_blocks;		/* check for bad blocks */
 };
 
 static char root_block[MINIX_BLOCK_SIZE];
@@ -152,8 +152,8 @@ static void __attribute__((__noreturn__)) usage(void)
 	fprintf(out, _(
 		"     --lock[=<mode>]     use exclusive device lock (%s, %s or %s)\n"), "yes", "no", "nonblock");
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(25));
-	printf(USAGE_MAN_TAIL("mkfs.minix(8)"));
+	fprintf(out, USAGE_HELP_OPTIONS(25));
+	fprintf(out, USAGE_MAN_TAIL("mkfs.minix(8)"));
 	exit(MKFS_EX_OK);
 }
 
