@@ -1,4 +1,11 @@
 /*
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
  * mkswap.c - set up a linux swap device
  *
  * Copyright (C) 1991 Linus Torvalds
@@ -8,6 +15,7 @@
  * Copyright (C) 2007-2014 Karel Zak <kzak@redhat.com>
  */
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -25,8 +33,10 @@
 # include <selinux/context.h>
 # include "selinux-utils.h"
 #endif
-#ifdef HAVE_LINUX_FIEMAP_H
+#ifdef HAVE_LINUX_FS_H
 # include <linux/fs.h>
+#endif
+#ifdef HAVE_LINUX_FIEMAP_H
 # include <linux/fiemap.h>
 #endif
 
@@ -41,6 +51,8 @@
 #include "c.h"
 #include "closestream.h"
 #include "ismounted.h"
+#include "optutils.h"
+#include "bitops.h"
 
 #ifdef HAVE_LIBUUID
 # include <uuid.h>
@@ -50,10 +62,24 @@
 # include <blkid.h>
 #endif
 
+#if defined(FS_IOC_FIEMAP) && defined(FIEMAP_EXTENT_LAST)
+# define USE_EXTENDS_CHECK 1
+#endif
+
+#if defined(FS_IOC_GETFLAGS) && defined(FS_NOCOW_FL)
+# define USE_NOCOW 1
+#endif
+
 #define MIN_GOODPAGES	10
 
 #define SELINUX_SWAPFILE_TYPE	"swapfile_t"
 
+enum ENDIANNESS {
+	ENDIANNESS_NATIVE,
+	ENDIANNESS_LITTLE,
+	ENDIANNESS_BIG,
+};
+
 struct mkswap_control {
 	struct swap_header_v1_2	*hdr;		/* swap header */
 	void			*signature_page;/* buffer with swap header */
@@ -68,17 +94,34 @@ struct mkswap_control {
 
 	int			user_pagesize;	/* --pagesize */
 	int			pagesize;	/* final pagesize used for the header */
+	off_t			offset;         /* offset of the header in the target */
 
 	char			*opt_label;	/* LABEL as specified on command line */
 	unsigned char		*uuid;		/* UUID parsed by libbuuid */
 
+	unsigned long long	filesz;		/* desired swap file size */
+
 	size_t			nbad_extents;
 
-	unsigned int		check:1,	/* --check */
-				verbose:1,      /* --verbose */
-				force:1;	/* --force */
+	enum ENDIANNESS         endianness;
+
+	bool			check,		/* --check */
+				verbose,      	/* --verbose */
+				quiet,        	/* --quiet */
+				force,		/* --force */
+				file;		/* --file */
 };
 
+static uint32_t cpu32_to_endianness(uint32_t v, enum ENDIANNESS e)
+{
+	switch (e) {
+		case ENDIANNESS_NATIVE: return v;
+		case ENDIANNESS_LITTLE: return cpu_to_le32(v);
+		case ENDIANNESS_BIG: return cpu_to_be32(v);
+	}
+	abort();
+}
+
 static void init_signature_page(struct mkswap_control *ctl)
 {
 	const int kernel_pagesize = getpagesize();
@@ -89,7 +132,7 @@ static void init_signature_page(struct mkswap_control *ctl)
 			errx(EXIT_FAILURE,
 			     _("Bad user-specified page size %u"),
 			       ctl->user_pagesize);
-		if (ctl->user_pagesize != kernel_pagesize)
+		if (!ctl->quiet && ctl->user_pagesize != kernel_pagesize)
 			warnx(_("Using user-specified page size %d, "
 				"instead of the system value %d"),
 				ctl->user_pagesize, kernel_pagesize);
@@ -130,12 +173,13 @@ static void set_uuid_and_label(const struct mkswap_control *ctl)
 	if (ctl->opt_label) {
 		xstrncpy(ctl->hdr->volume_name,
 			 ctl->opt_label, sizeof(ctl->hdr->volume_name));
-		if (strlen(ctl->opt_label) > strlen(ctl->hdr->volume_name))
+		if (!ctl->quiet
+		    && strlen(ctl->opt_label) > strlen(ctl->hdr->volume_name))
 			warnx(_("Label was truncated."));
 	}
 
 	/* report results */
-	if (ctl->uuid || ctl->opt_label) {
+	if (!ctl->quiet && (ctl->uuid || ctl->opt_label)) {
 		if (ctl->opt_label)
 			printf("LABEL=%s, ", ctl->hdr->volume_name);
 		else
@@ -164,17 +208,25 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -c, --check               check bad blocks before creating the swap area\n"), out);
 	fputs(_(" -f, --force               allow swap size area be larger than device\n"), out);
+	fputs(_(" -q, --quiet               suppress output and warning messages\n"), out);
 	fputs(_(" -p, --pagesize SIZE       specify page size in bytes\n"), out);
 	fputs(_(" -L, --label LABEL         specify label\n"), out);
 	fputs(_(" -v, --swapversion NUM     specify swap-space version number\n"), out);
 	fputs(_(" -U, --uuid UUID           specify the uuid to use\n"), out);
+	fprintf(out,
+	      _(" -e, --endianness=<value>  specify the endianness to use "
+	                                    "(%s, %s or %s)\n"), "native", "little", "big");
+	fputs(_(" -o, --offset OFFSET       specify the offset in the device\n"), out);
+	fputs(_(" -s, --size SIZE           specify the size of a swap file in bytes\n"), out);
+	fputs(_(" -F, --file                create a swap file\n"), out);
 	fputs(_("     --verbose             verbose output\n"), out);
 
 	fprintf(out,
 	      _("     --lock[=<mode>]       use exclusive device lock (%s, %s or %s)\n"), "yes", "no", "nonblock");
-	printf(USAGE_HELP_OPTIONS(27));
 
-	printf(USAGE_MAN_TAIL("mkswap(8)"));
+	fprintf(out, USAGE_HELP_OPTIONS(27));
+
+	fprintf(out, USAGE_MAN_TAIL("mkswap(8)"));
 	exit(EXIT_SUCCESS);
 }
 
@@ -213,12 +265,14 @@ static void check_blocks(struct mkswap_control *ctl)
 			page_bad(ctl, current_page);
 		current_page++;
 	}
-	printf(P_("%lu bad page\n", "%lu bad pages\n", ctl->nbadpages), ctl->nbadpages);
+
+	if (!ctl->quiet)
+		printf(P_("%lu bad page\n", "%lu bad pages\n", ctl->nbadpages), ctl->nbadpages);
 	free(buffer);
 }
 
 
-#ifdef HAVE_LINUX_FIEMAP_H
+#ifdef USE_EXTENDS_CHECK
 static void warn_extent(struct mkswap_control *ctl, const char *msg, uint64_t off)
 {
 	if (ctl->nbad_extents == 0) {
@@ -285,7 +339,7 @@ static void check_extents(struct mkswap_control *ctl)
 				warn_extent(ctl, _("data inline extent at offset %ju"),
 						(uintmax_t) e->fe_logical);
 			if (e->fe_flags & FIEMAP_EXTENT_SHARED)
-				 warn_extent(ctl, _("shared extent at offset %ju"),
+				warn_extent(ctl, _("shared extent at offset %ju"),
 						(uintmax_t) e->fe_logical);
 			if (e->fe_flags & FIEMAP_EXTENT_DELALLOC)
 				warn_extent(ctl, _("unallocated extent at offset %ju"),
@@ -305,21 +359,27 @@ done:
 	if (ctl->nbad_extents)
 		fputc('\n', stderr);
 }
-#endif /* HAVE_LINUX_FIEMAP_H */
+#endif /* USE_EXTENDS_CHECK */
 
 /* return size in pages */
 static unsigned long long get_size(const struct mkswap_control *ctl)
 {
-	int fd;
 	unsigned long long size;
 
-	fd = open(ctl->devname, O_RDONLY);
-	if (fd < 0)
-		err(EXIT_FAILURE, _("cannot open %s"), ctl->devname);
-	if (blkdev_get_size(fd, &size) == 0)
-		size /= ctl->pagesize;
-
-	close(fd);
+	if (ctl->file && ctl->filesz)
+		size = ctl->filesz;
+	else {
+		int fd = open(ctl->devname, O_RDONLY);
+		if (fd < 0)
+			err(EXIT_FAILURE, _("cannot open %s"), ctl->devname);
+		if (blkdev_get_size(fd, &size) < 0)
+			err(EXIT_FAILURE, _("cannot determine size of %s"), ctl->devname);
+		if ((unsigned long long) ctl->offset > size)
+			errx(EXIT_FAILURE, _("offset larger than file size"));
+		close(fd);
+	}
+	size -= ctl->offset;
+	size /= ctl->pagesize;
 	return size;
 }
 
@@ -340,19 +400,55 @@ static void open_device(struct mkswap_control *ctl)
 	assert(ctl);
 	assert(ctl->devname);
 
-	if (stat(ctl->devname, &ctl->devstat) < 0)
-		err(EXIT_FAILURE, _("stat of %s failed"), ctl->devname);
-	ctl->fd = open_blkdev_or_file(&ctl->devstat, ctl->devname, O_RDWR);
+	if (ctl->file) {
+		if (stat(ctl->devname, &ctl->devstat) == 0) {
+			if (!S_ISREG(ctl->devstat.st_mode))
+				err(EXIT_FAILURE, _("cannot create swap file %s: node isn't regular file"), ctl->devname);
+			if (chmod(ctl->devname, 0600) < 9)
+				err(EXIT_FAILURE, _("cannot set permissions on swap file %s"), ctl->devname);
+		}
+		ctl->fd = open(ctl->devname, O_RDWR | O_CREAT, 0600);
+	} else {
+		if (stat(ctl->devname, &ctl->devstat) < 0)
+			err(EXIT_FAILURE, _("stat of %s failed"), ctl->devname);
+		ctl->fd = open_blkdev_or_file(&ctl->devstat, ctl->devname, O_RDWR);
+	}
 	if (ctl->fd < 0)
 		err(EXIT_FAILURE, _("cannot open %s"), ctl->devname);
 
+	if (ctl->file && ctl->filesz) {
+#ifdef USE_NOCOW
+		/* Let's attempt to set the "nocow" attribute for Btrfs, etc.
+		 * Ignore if unsupported. */
+		int attr = 0;
+
+		if (ioctl(ctl->fd, FS_IOC_GETFLAGS, &attr) == 0) {
+			attr |= FS_NOCOW_FL;
+			if (ioctl(ctl->fd, FS_IOC_SETFLAGS, &attr) < 0 &&
+			    (errno != EOPNOTSUPP && errno != ENOSYS && errno != EINVAL))
+				warn(_("failed to set 'nocow' attribute"));
+		}
+#endif
+		if (ftruncate(ctl->fd, ctl->filesz) < 0)
+			err(EXIT_FAILURE, _("couldn't allocate swap file %s"), ctl->devname);
+#ifdef HAVE_POSIX_FALLOCATE
+		errno = posix_fallocate(ctl->fd, 0, ctl->filesz);
+		if (errno)
+			err(EXIT_FAILURE, _("couldn't allocate swap file %s"), ctl->devname);
+#elif defined(HAVE_FALLOCATE)
+		if (fallocate(ctl->fd, 0, 0, ctl->filesz) < 0)
+			err(EXIT_FAILURE, _("couldn't allocate swap file %s"), ctl->devname);
+#endif
+	}
+
 	if (blkdev_lock(ctl->fd, ctl->devname, ctl->lockmode) != 0)
 		exit(EXIT_FAILURE);
 
 	if (ctl->check && S_ISREG(ctl->devstat.st_mode)) {
 		ctl->check = 0;
-		warnx(_("warning: checking bad blocks from swap file is not supported: %s"),
-		       ctl->devname);
+		if (!ctl->quiet)
+			warnx(_("warning: checking bad blocks from swap file is not supported: %s"),
+			       ctl->devname);
 	}
 }
 
@@ -409,12 +505,13 @@ static void wipe_device(struct mkswap_control *ctl)
 		while (blkid_do_probe(pr) == 0) {
 			const char *data = NULL;
 
-			if (blkid_probe_lookup_value(pr, "TYPE", &data, NULL) == 0 && data)
+			if (!ctl->quiet
+			    && blkid_probe_lookup_value(pr, "TYPE", &data, NULL) == 0 && data)
 				warnx(_("%s: warning: wiping old %s signature."), ctl->devname, data);
 			blkid_do_wipe(pr, 0);
 		}
 #endif
-	} else {
+	} else if (!ctl->quiet) {
 		warnx(_("%s: warning: don't erase bootbits sectors"),
 			ctl->devname);
 		if (type)
@@ -433,11 +530,15 @@ static void wipe_device(struct mkswap_control *ctl)
 
 static void write_header_to_device(struct mkswap_control *ctl)
 {
+	off_t offset;
+
 	assert(ctl);
 	assert(ctl->fd > -1);
 	assert(ctl->signature_page);
 
-	if (lseek(ctl->fd, SIGNATURE_OFFSET, SEEK_SET) != SIGNATURE_OFFSET)
+	offset = SIGNATURE_OFFSET + ctl->offset;
+
+	if (lseek(ctl->fd, offset, SEEK_SET) != offset)
 		errx(EXIT_FAILURE, _("unable to rewind swap-device"));
 
 	if (write_all(ctl->fd, (char *) ctl->signature_page + SIGNATURE_OFFSET,
@@ -449,7 +550,7 @@ static void write_header_to_device(struct mkswap_control *ctl)
 
 int main(int argc, char **argv)
 {
-	struct mkswap_control ctl = { .fd = -1 };
+	struct mkswap_control ctl = { .fd = -1, .endianness = ENDIANNESS_NATIVE };
 	int c, permMask;
 	uint64_t sz;
 	int version = SWAP_VERSION;
@@ -465,10 +566,15 @@ int main(int argc, char **argv)
 	static const struct option longopts[] = {
 		{ "check",       no_argument,       NULL, 'c' },
 		{ "force",       no_argument,       NULL, 'f' },
+		{ "quiet",       no_argument,       NULL, 'q' },
 		{ "pagesize",    required_argument, NULL, 'p' },
 		{ "label",       required_argument, NULL, 'L' },
 		{ "swapversion", required_argument, NULL, 'v' },
 		{ "uuid",        required_argument, NULL, 'U' },
+		{ "endianness",  required_argument, NULL, 'e' },
+		{ "offset",      required_argument, NULL, 'o' },
+		{ "size",        required_argument, NULL, 's' },
+		{ "file",        no_argument,       NULL, 'F' },
 		{ "version",     no_argument,       NULL, 'V' },
 		{ "help",        no_argument,       NULL, 'h' },
 		{ "lock",        optional_argument, NULL, OPT_LOCK },
@@ -476,12 +582,21 @@ int main(int argc, char **argv)
 		{ NULL,          0, NULL, 0 }
 	};
 
+	static const ul_excl_t excl[] = {       /* rows and cols in ASCII order */
+		{ 'c', 'q' },
+		{ 0 }
+	};
+	int excl_st[ARRAY_SIZE(excl)] = UL_EXCL_STATUS_INIT;
+
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	close_stdout_atexit();
 
-	while((c = getopt_long(argc, argv, "cfp:L:v:U:Vh", longopts, NULL)) != -1) {
+	while((c = getopt_long(argc, argv, "cfp:qL:v:U:e:o:s:FVh", longopts, NULL)) != -1) {
+
+		err_exclusive_options(c, longopts, excl, excl_st);
+
 		switch (c) {
 		case 'c':
 			ctl.check = 1;
@@ -492,6 +607,9 @@ int main(int argc, char **argv)
 		case 'p':
 			ctl.user_pagesize = strtou32_or_err(optarg, _("parsing page size failed"));
 			break;
+		case 'q':
+			ctl.quiet = 1;
+			break;
 		case 'L':
 			ctl.opt_label = optarg;
 			break;
@@ -509,9 +627,53 @@ int main(int argc, char **argv)
 				program_invocation_short_name);
 #endif
 			break;
-		case 'V':
-			print_version(EXIT_SUCCESS);
+		case 'e':
+			if (strcmp(optarg, "native") == 0) {
+				ctl.endianness = ENDIANNESS_NATIVE;
+			} else if (strcmp(optarg, "little") == 0) {
+				ctl.endianness = ENDIANNESS_LITTLE;
+			} else if (strcmp(optarg, "big") == 0) {
+				ctl.endianness = ENDIANNESS_BIG;
+			} else {
+				errx(EXIT_FAILURE,
+					_("invalid endianness %s is not supported"), optarg);
+			}
 			break;
+		case 'o':
+			ctl.offset = str2unum_or_err(optarg,
+					10, _("Invalid offset"), SINT_MAX(off_t));
+			break;
+		case 's':
+			ctl.filesz = strtosize_or_err(optarg, _("Invalid size"));
+			break;
+		case 'F':
+			ctl.file = 1;
+			break;
+		case 'V':
+		{
+			static const char *const features[] = {
+#ifdef USE_EXTENDS_CHECK
+				"extends-check",
+#endif
+#ifdef USE_NOCOW
+				"nocow",
+#endif
+#if defined(HAVE_POSIX_FALLOCATE) || defined(HAVE_FALLOCATE)
+				"fallocate",
+#endif
+#ifdef HAVE_LIBBLKID
+				"blkid-check",
+#endif
+#ifdef HAVE_LIBUUID
+				"uuid",
+#endif
+#ifdef HAVE_LIBSELINUX
+				"selinux",
+#endif
+				NULL,
+			};
+			print_version_with_features(EXIT_SUCCESS, features);
+		}
 		case OPT_LOCK:
 			ctl.lockmode = "1";
 			if (optarg) {
@@ -541,7 +703,13 @@ int main(int argc, char **argv)
 
 #ifdef HAVE_LIBUUID
 	if(opt_uuid) {
-		if (uuid_parse(opt_uuid, uuid_dat) != 0)
+		if (strcmp(opt_uuid, "clear") == 0)
+			uuid_clear(uuid_dat);
+		else if (strcmp(opt_uuid, "random") == 0)
+			uuid_generate_random(uuid_dat);
+		else if (strcmp(opt_uuid, "time") == 0)
+			uuid_generate_time(uuid_dat);
+		else if (uuid_parse(opt_uuid, uuid_dat) != 0)
 			errx(EXIT_FAILURE, _("error: parsing UUID failed"));
 	} else
 		uuid_generate(uuid_dat);
@@ -577,8 +745,9 @@ int main(int argc, char **argv)
 	if (ctl.npages > UINT32_MAX) {
 		/* true when swap is bigger than 17.59 terabytes */
 		ctl.npages = UINT32_MAX;
-		warnx(_("warning: truncating swap area to %llu KiB"),
-			ctl.npages * ctl.pagesize / 1024);
+		if (!ctl.quiet)
+			warnx(_("warning: truncating swap area to %llu KiB"),
+				ctl.npages * ctl.pagesize / 1024);
 	}
 
 	if (is_mounted(ctl.devname))
@@ -588,28 +757,29 @@ int main(int argc, char **argv)
 
 	open_device(&ctl);
 	permMask = S_ISBLK(ctl.devstat.st_mode) ? 07007 : 07077;
-	if ((ctl.devstat.st_mode & permMask) != 0)
+	if (!ctl.quiet && (ctl.devstat.st_mode & permMask) != 0)
 		warnx(_("%s: insecure permissions %04o, fix with: chmod %04o %s"),
 			ctl.devname, ctl.devstat.st_mode & 07777,
 			~permMask & 0666, ctl.devname);
-	if (getuid() == 0 && S_ISREG(ctl.devstat.st_mode) && ctl.devstat.st_uid != 0)
+	if (!ctl.quiet
+	    && getuid() == 0 && S_ISREG(ctl.devstat.st_mode) && ctl.devstat.st_uid != 0)
 		warnx(_("%s: insecure file owner %d, fix with: chown 0:0 %s"),
 			ctl.devname, ctl.devstat.st_uid, ctl.devname);
 
 
 	if (ctl.check)
 		check_blocks(&ctl);
-#ifdef HAVE_LINUX_FIEMAP_H
-	if (S_ISREG(ctl.devstat.st_mode))
+#ifdef USE_EXTENDS_CHECK
+	if (!ctl.quiet && S_ISREG(ctl.devstat.st_mode))
 		check_extents(&ctl);
 #endif
 
 	wipe_device(&ctl);
 
 	assert(ctl.hdr);
-	ctl.hdr->version = version;
-	ctl.hdr->last_page = ctl.npages - 1;
-	ctl.hdr->nr_badpages = ctl.nbadpages;
+	ctl.hdr->version = cpu32_to_endianness(version, ctl.endianness);
+	ctl.hdr->last_page = cpu32_to_endianness(ctl.npages - 1, ctl.endianness);
+	ctl.hdr->nr_badpages = cpu32_to_endianness(ctl.nbadpages, ctl.endianness);
 
 	if ((ctl.npages - MIN_GOODPAGES) < ctl.nbadpages)
 		errx(EXIT_FAILURE, _("Unable to set up swap-space: unreadable"));
@@ -617,8 +787,9 @@ int main(int argc, char **argv)
 	sz = (ctl.npages - ctl.nbadpages - 1) * ctl.pagesize;
 	strsz = size_to_human_string(SIZE_SUFFIX_SPACE | SIZE_SUFFIX_3LETTER, sz);
 
-	printf(_("Setting up swapspace version %d, size = %s (%"PRIu64" bytes)\n"),
-		version, strsz, sz);
+	if (!ctl.quiet)
+		printf(_("Setting up swapspace version %d, size = %s (%"PRIu64" bytes)\n"),
+			version, strsz, sz);
 	free(strsz);
 
 	set_signature(&ctl);
@@ -629,8 +800,10 @@ int main(int argc, char **argv)
 	deinit_signature_page(&ctl);
 
 #ifdef HAVE_LIBSELINUX
-	if (S_ISREG(ctl.devstat.st_mode) && is_selinux_enabled() > 0) {
-		char *context_string, *oldcontext;
+	if ((ctl.file || S_ISREG(ctl.devstat.st_mode)) &&
+            is_selinux_enabled() > 0) {
+		const char *context_string;
+		char *oldcontext;
 		context_t newcontext;
 
 		if (fgetfilecon(ctl.fd, &oldcontext) < 0) {
