@@ -1,3 +1,7 @@
+/*
+ * No copyright is claimed. This code is in the public domain; do with it what
+ * you wish.
+ */
 #ifndef UTIL_LINUX_MD5_H
 #define UTIL_LINUX_MD5_H
 
@@ -11,9 +15,9 @@ struct UL_MD5Context {
 	unsigned char in[64];
 };
 
-void ul_MD5Init(struct UL_MD5Context *context);
-void ul_MD5Update(struct UL_MD5Context *context, unsigned char const *buf, unsigned len);
-void ul_MD5Final(unsigned char digest[UL_MD5LENGTH], struct UL_MD5Context *context);
+void ul_MD5Init(struct UL_MD5Context *ctx);
+void ul_MD5Update(struct UL_MD5Context *ctx, unsigned char const *buf, unsigned len);
+void ul_MD5Final(unsigned char digest[UL_MD5LENGTH], struct UL_MD5Context *ctx);
 void ul_MD5Transform(uint32_t buf[4], uint32_t const in[16]);
 
 /*
