@@ -13,9 +13,11 @@
 
 #include <sys/time.h>
 
+#include "timeutils.h"
+
 extern int get_boot_time(struct timeval *boot_time);
 
-extern time_t get_suspended_time(void);
+extern usec_t get_suspended_time(void);
 
 extern int gettime_monotonic(struct timeval *tv);
 
