@@ -232,6 +232,9 @@ DEF_GUID("49F48DAA-B10E-11DC-B99B-0019D1879648", N_("NetBSD RAID")),
 DEF_GUID("FE3A2A5D-4F32-41A7-B725-ACCC3285A309", N_("ChromeOS kernel")),
 DEF_GUID("3CB8E202-3B7E-47DD-8A3C-7FF2A13CFCEC", N_("ChromeOS root fs")),
 DEF_GUID("2E0A753D-9E48-43B0-8337-B15192CB1B5E", N_("ChromeOS reserved")),
+DEF_GUID("CAB6E88E-ABF3-4102-A07A-D4BB9BE3C1D3", N_("ChromeOS firmware")),
+DEF_GUID("09845860-705F-4BB5-B16C-8A8A099CAF52", N_("ChromeOS miniOS")),
+DEF_GUID("3F0F8318-F146-4E6B-8222-C28C8F02E0D5", N_("ChromeOS hibernate")),
 
 /* MidnightBSD */
 DEF_GUID("85D5E45A-237C-11E1-B4B3-E89A8F7FC3A7", N_("MidnightBSD data")),
@@ -286,3 +289,13 @@ DEF_GUID("3D48CE54-1D16-11DC-8696-01301BB8A9F5", N_("DragonFlyBSD Label64")),
 DEF_GUID("BD215AB2-1D16-11DC-8696-01301BB8A9F5", N_("DragonFlyBSD Legacy")),
 DEF_GUID("61DC63AC-6E38-11DC-8513-01301BB8A9F5", N_("DragonFlyBSD HAMMER")),
 DEF_GUID("5CBB9AD1-862D-11DC-A94D-01301BB8A9F5", N_("DragonFlyBSD HAMMER2")),
+
+/* U-Boot */
+DEF_GUID("3DE21764-95BD-54BD-A5C3-4ABE786F38A8", N_("U-Boot environment")),
+
+/* Atari TOS */
+DEF_GUID("734E5AFE-F61A-11E6-BC64-92361F002671", N_("Atari TOS basic data")),
+DEF_GUID("35540011-B055-499F-842D-C69AECA357B7", N_("Atari TOS raw data (XHDI)")),
+
+/* Minix */
+DEF_GUID("481B2A38-0561-420B-B72A-F1C4988EFC16", N_("Minix filesystem")),
