@@ -1,8 +1,8 @@
 /*
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
- * This file may be redistributed under the terms of the GNU Public
- * License.
+ * This file may be redistributed under the terms of the
+ * GNU General Public License.
  *
  * Based on linux-perf/git scm
  *
@@ -11,6 +11,7 @@
  */
 
 #include <unistd.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <err.h>
@@ -44,7 +45,7 @@ struct child_process {
 	struct sigaction orig_sigquit;
 	struct sigaction orig_sigpipe;
 
-	unsigned no_stdin:1;
+	bool no_stdin;
 	void (*preexec_cb)(void);
 };
 static struct child_process pager_process;
@@ -171,6 +172,7 @@ static void wait_for_pager(void)
 
 static void wait_for_pager_signal(int signo)
 {
+	UL_PROTECT_ERRNO;
 	wait_for_pager();
 	raise(signo);
 }
