@@ -1,6 +1,8 @@
 /*
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Copyright 2010 Lennart Poettering
+ * Copyright (C) 2010 Lennart Poettering
+ * Copyright (C) 2015-2022 Karel Zak <kzak@redhat.com>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +24,7 @@
 #include "strutils.h"
 #include "strv.h"
 
-void strv_clear(char **l) {
+static void ul_strv_clear(char **l) {
         char **k;
 
         if (!l)
@@ -34,16 +36,16 @@ void strv_clear(char **l) {
         *l = NULL;
 }
 
-char **strv_free(char **l) {
-        strv_clear(l);
+char **ul_strv_free(char **l) {
+        ul_strv_clear(l);
         free(l);
         return NULL;
 }
 
-char **strv_copy(char * const *l) {
+char **ul_strv_copy(char * const *l) {
         char **r, **k;
 
-        k = r = malloc(sizeof(char *) * (strv_length(l) + 1));
+        k = r = malloc(sizeof(char *) * (ul_strv_length(l) + 1));
         if (!r)
                 return NULL;
 
@@ -51,7 +53,7 @@ char **strv_copy(char * const *l) {
                 for (; *l; k++, l++) {
                         *k = strdup(*l);
                         if (!*k) {
-                                strv_free(r);
+                                ul_strv_free(r);
                                 return NULL;
                         }
                 }
@@ -60,7 +62,7 @@ char **strv_copy(char * const *l) {
         return r;
 }
 
-unsigned strv_length(char * const *l) {
+unsigned ul_strv_length(char * const *l) {
         unsigned n = 0;
 
         if (!l)
@@ -72,7 +74,7 @@ unsigned strv_length(char * const *l) {
         return n;
 }
 
-char **strv_new_ap(const char *x, va_list ap) {
+static char **ul_strv_new_ap(const char *x, va_list ap) {
         const char *s;
         char **a;
         unsigned n = 0, i = 0;
@@ -80,7 +82,7 @@ char **strv_new_ap(const char *x, va_list ap) {
 
         /* As a special trick we ignore all listed strings that equal
          * (const char*) -1. This is supposed to be used with the
-         * STRV_IFNOTNULL() macro to include possibly NULL strings in
+         * UL_STRV_IFNOTNULL() macro to include possibly NULL strings in
          * the string list. */
 
         if (x) {
@@ -127,27 +129,27 @@ char **strv_new_ap(const char *x, va_list ap) {
         return a;
 
 fail:
-        strv_free(a);
+        ul_strv_free(a);
         return NULL;
 }
 
-char **strv_new(const char *x, ...) {
+char **ul_strv_new(const char *x, ...) {
         char **r;
         va_list ap;
 
         va_start(ap, x);
-        r = strv_new_ap(x, ap);
+        r = ul_strv_new_ap(x, ap);
         va_end(ap);
 
         return r;
 }
 
-int strv_extend_strv(char ***a, char **b) {
+int ul_strv_extend_strv(char ***a, char **b) {
         int r;
         char **s;
 
-        STRV_FOREACH(s, b) {
-                r = strv_extend(a, *s);
+        UL_STRV_FOREACH(s, b) {
+                r = ul_strv_extend(a, *s);
                 if (r < 0)
                         return r;
         }
@@ -155,18 +157,18 @@ int strv_extend_strv(char ***a, char **b) {
         return 0;
 }
 
-int strv_extend_strv_concat(char ***a, char **b, const char *suffix) {
+int ul_strv_extend_strv_concat(char ***a, char **b, const char *suffix) {
         int r;
         char **s;
 
-        STRV_FOREACH(s, b) {
+        UL_STRV_FOREACH(s, b) {
                 char *v;
 
-                v = strappend(*s, suffix);
+                v = ul_strconcat(*s, suffix);
                 if (!v)
                         return -ENOMEM;
 
-                r = strv_push(a, v);
+                r = ul_strv_push(a, v);
                 if (r < 0) {
                         free(v);
                         return r;
@@ -178,13 +180,13 @@ int strv_extend_strv_concat(char ***a, char **b, const char *suffix) {
 
 
 #define _FOREACH_WORD(word, length, s, separator, quoted, state)        \
-        for ((state) = (s), (word) = split(&(state), &(length), (separator), (quoted)); (word); (word) = split(&(state), &(length), (separator), (quoted)))
+        for ((state) = (s), (word) = ul_split(&(state), &(length), (separator), (quoted)); (word); (word) = ul_split(&(state), &(length), (separator), (quoted)))
 
 #define FOREACH_WORD_SEPARATOR(word, length, s, separator, state)       \
         _FOREACH_WORD(word, length, s, separator, false, state)
 
 
-char **strv_split(const char *s, const char *separator) {
+char **ul_strv_split(const char *s, const char *separator) {
         const char *word, *state;
         size_t l;
         unsigned n, i;
@@ -204,7 +206,7 @@ char **strv_split(const char *s, const char *separator) {
         FOREACH_WORD_SEPARATOR(word, l, s, separator, state) {
                 r[i] = strndup(word, l);
                 if (!r[i]) {
-                        strv_free(r);
+                        ul_strv_free(r);
                         return NULL;
                 }
 
@@ -215,7 +217,7 @@ char **strv_split(const char *s, const char *separator) {
         return r;
 }
 
-char *strv_join(char **l, const char *separator) {
+char *ul_strv_join(char **l, const char *separator) {
         char *r, *e;
         char **s;
         size_t n, k;
@@ -226,7 +228,7 @@ char *strv_join(char **l, const char *separator) {
         k = strlen(separator);
 
         n = 0;
-        STRV_FOREACH(s, l) {
+        UL_STRV_FOREACH(s, l) {
                 if (n != 0)
                         n += k;
                 n += strlen(*s);
@@ -237,7 +239,7 @@ char *strv_join(char **l, const char *separator) {
                 return NULL;
 
         e = r;
-        STRV_FOREACH(s, l) {
+        UL_STRV_FOREACH(s, l) {
                 if (e != r)
                         e = stpcpy(e, separator);
 
@@ -249,21 +251,21 @@ char *strv_join(char **l, const char *separator) {
         return r;
 }
 
-int strv_push(char ***l, char *value) {
+int ul_strv_push(char ***l, char *value) {
         char **c;
         unsigned n, m;
 
         if (!value)
                 return 0;
 
-        n = strv_length(*l);
+        n = ul_strv_length(*l);
 
         /* Increase and check for overflow */
         m = n + 2;
         if (m < n)
                 return -ENOMEM;
 
-        c = realloc(*l, sizeof(char *) * m);
+        c = reallocarray(*l, m, sizeof(char *));
         if (!c)
                 return -ENOMEM;
 
@@ -274,14 +276,14 @@ int strv_push(char ***l, char *value) {
         return 0;
 }
 
-int strv_push_prepend(char ***l, char *value) {
+int ul_strv_push_prepend(char ***l, char *value) {
         char **c;
         unsigned n, m, i;
 
         if (!value)
                 return 0;
 
-        n = strv_length(*l);
+        n = ul_strv_length(*l);
 
         /* increase and check for overflow */
         m = n + 2;
@@ -304,27 +306,27 @@ int strv_push_prepend(char ***l, char *value) {
         return 0;
 }
 
-int strv_consume(char ***l, char *value) {
+int ul_strv_consume(char ***l, char *value) {
         int r;
 
-        r = strv_push(l, value);
+        r = ul_strv_push(l, value);
         if (r < 0)
                 free(value);
 
         return r;
 }
 
-int strv_consume_prepend(char ***l, char *value) {
+int ul_strv_consume_prepend(char ***l, char *value) {
         int r;
 
-        r = strv_push_prepend(l, value);
+        r = ul_strv_push_prepend(l, value);
         if (r < 0)
                 free(value);
 
         return r;
 }
 
-int strv_extend(char ***l, const char *value) {
+int ul_strv_extend(char ***l, const char *value) {
         char *v;
 
         if (!value)
@@ -334,10 +336,10 @@ int strv_extend(char ***l, const char *value) {
         if (!v)
                 return -ENOMEM;
 
-        return strv_consume(l, v);
+        return ul_strv_consume(l, v);
 }
 
-char **strv_remove(char **l, const char *s) {
+char **ul_strv_remove(char **l, const char *s) {
         char **f, **t;
 
         if (!l)
@@ -358,7 +360,7 @@ char **strv_remove(char **l, const char *s) {
         return l;
 }
 
-int strv_extendf(char ***l, const char *format, ...) {
+int ul_strv_extendf(char ***l, const char *format, ...) {
         va_list ap;
         char *x;
         int r;
@@ -370,10 +372,10 @@ int strv_extendf(char ***l, const char *format, ...) {
         if (r < 0)
                 return -ENOMEM;
 
-        return strv_consume(l, x);
+        return ul_strv_consume(l, x);
 }
 
-int strv_extendv(char ***l, const char *format, va_list ap) {
+int ul_strv_extendv(char ***l, const char *format, va_list ap) {
         char *x;
         int r;
 
@@ -381,13 +383,13 @@ int strv_extendv(char ***l, const char *format, va_list ap) {
         if (r < 0)
                 return -ENOMEM;
 
-        return strv_consume(l, x);
+        return ul_strv_consume(l, x);
 }
 
-char **strv_reverse(char **l) {
+char **ul_strv_reverse(char **l) {
         unsigned n, i;
 
-        n = strv_length(l);
+        n = ul_strv_length(l);
         if (n <= 1)
                 return l;
 
