@@ -125,7 +125,7 @@
 /*
  * All supported chains
  */
-static const struct blkid_chaindrv *chains_drvs[] = {
+static const struct blkid_chaindrv *const chains_drvs[] = {
 	[BLKID_CHAIN_SUBLKS] = &superblocks_drv,
 	[BLKID_CHAIN_TOPLGY] = &topology_drv,
 	[BLKID_CHAIN_PARTS] = &partitions_drv
@@ -603,20 +603,6 @@ static struct blkid_bufinfo *read_buffer(blkid_probe pr, uint64_t real_off, uint
 		 * audio+data disks */
 		if (ret >= 0 || blkid_probe_is_cdrom(pr) || blkdid_probe_is_opal_locked(pr))
 			errno = 0;
-#ifdef HAVE_OPAL_GET_STATUS
-		else {
-			struct opal_status st = { };
-
-			/* If the device is locked with OPAL, we'll fail to read with I/O
-			 * errors when probing deep into the block device. Do not return
-			 * an error, so that we can move on to different types of checks.*/
-			ret = ioctl(pr->fd, IOC_OPAL_GET_STATUS, &st);
-			if (ret == 0 && (st.flags & OPAL_FL_LOCKED)) {
-				pr->flags |= BLKID_FL_OPAL_LOCKED;
-				errno = 0;
-			}
-		}
-#endif
 
 		return NULL;
 	}
@@ -805,6 +791,35 @@ const unsigned char *blkid_probe_get_buffer(blkid_probe pr, uint64_t off, uint64
 	return real_off ? bf->data + (real_off - bf->off + bias) : bf->data + bias;
 }
 
+#ifdef O_DIRECT
+/*
+ * This is blkid_probe_get_buffer with the read done as an O_DIRECT operation.
+ * Note that @off is offset within probing area, the probing area is defined by
+ * pr->off and pr->size.
+ */
+const unsigned char *blkid_probe_get_buffer_direct(blkid_probe pr, uint64_t off, uint64_t len)
+{
+	const unsigned char *ret = NULL;
+	int flags, rc, olderrno;
+
+	flags = fcntl(pr->fd, F_GETFL);
+	rc = fcntl(pr->fd, F_SETFL, flags | O_DIRECT);
+	if (rc) {
+		DBG(LOWPROBE, ul_debug("fcntl F_SETFL failed to set O_DIRECT"));
+		errno = 0;
+		return NULL;
+	}
+	ret = blkid_probe_get_buffer(pr, off, len);
+	olderrno = errno;
+	rc = fcntl(pr->fd, F_SETFL, flags);
+	if (rc) {
+		DBG(LOWPROBE, ul_debug("fcntl F_SETFL failed to clear O_DIRECT"));
+		errno = olderrno;
+	}
+	return ret;
+}
+#endif
+
 /**
  * blkid_probe_reset_buffers:
  * @pr: prober
@@ -911,6 +926,25 @@ int blkid_probe_is_cdrom(blkid_probe pr)
 
 int blkdid_probe_is_opal_locked(blkid_probe pr)
 {
+	if (!(pr->flags & BLKID_FL_OPAL_CHECKED)) {
+		pr->flags |= BLKID_FL_OPAL_CHECKED;
+
+#ifdef HAVE_OPAL_GET_STATUS
+		ssize_t ret;
+		struct opal_status st = { };
+		int errsv = errno;
+
+		/* If the device is locked with OPAL, we'll fail to read with I/O
+		 * errors when probing deep into the block device. */
+		ret = ioctl(pr->fd, IOC_OPAL_GET_STATUS, &st);
+		if (ret == 0 && (st.flags & OPAL_FL_LOCKED)) {
+			pr->flags |= BLKID_FL_OPAL_LOCKED;
+		}
+
+		errno = errsv;
+#endif
+	}
+
 	return (pr->flags & BLKID_FL_OPAL_LOCKED);
 }
 
@@ -1982,11 +2016,11 @@ static void blkid_probe_log_csum_mismatch(blkid_probe pr, size_t n, const void *
 		sprintf(&expected_hex[i], "%02X", ((const unsigned char *) expected)[i / 2]);
 	}
 
-	ul_debug(
+	DBG(LOWPROBE, ul_debug(
 		"incorrect checksum for type %s,"
 		" got %*s, expected %*s",
 		blkid_probe_get_probername(pr),
-		hex_size, csum_hex, hex_size, expected_hex);
+		hex_size, csum_hex, hex_size, expected_hex));
 }
 
 
