@@ -43,7 +43,9 @@ struct vfat_super_block {
 /* 30*/	uint16_t	vs_fsinfo_sector;
 /* 32*/	uint16_t	vs_backup_boot;
 /* 34*/	uint16_t	vs_reserved2[6];
-/* 40*/	unsigned char	vs_unknown[3];
+/* 40*/	unsigned char	vs_drive_number;
+/* 41*/	unsigned char	vs_boot_flags;
+/* 42*/	unsigned char	vs_ext_boot_sign; /* 0x28 - without vs_label/vs_magic; 0x29 - with */
 /* 43*/	unsigned char	vs_serno[4];
 /* 47*/	unsigned char	vs_label[11];
 /* 52*/	unsigned char   vs_magic[8];
@@ -53,6 +55,7 @@ struct vfat_super_block {
 
 /* Yucky misaligned values */
 struct msdos_super_block {
+/* DOS 2.0 BPB */
 /* 00*/	unsigned char	ms_ignored[3];
 /* 03*/	unsigned char	ms_sysid[8];
 /* 0b*/	unsigned char	ms_sector_size[2];
@@ -63,16 +66,21 @@ struct msdos_super_block {
 /* 13*/	unsigned char	ms_sectors[2]; /* =0 iff V3 or later */
 /* 15*/	unsigned char	ms_media;
 /* 16*/	uint16_t	ms_fat_length; /* Sectors per FAT */
+/* DOS 3.0 BPB */
 /* 18*/	uint16_t	ms_secs_track;
 /* 1a*/	uint16_t	ms_heads;
 /* 1c*/	uint32_t	ms_hidden;
-/* V3 BPB */
+/* DOS 3.31 BPB */
 /* 20*/	uint32_t	ms_total_sect; /* iff ms_sectors == 0 */
-/* V4 BPB */
-/* 24*/	unsigned char	ms_unknown[3]; /* Phys drive no., resvd, V4 sig (0x29) */
+/* DOS 3.4 EBPB */
+/* 24*/	unsigned char	ms_drive_number;
+/* 25*/	unsigned char	ms_boot_flags;
+/* 26*/	unsigned char	ms_ext_boot_sign; /* 0x28 - DOS 3.4 EBPB; 0x29 - DOS 4.0 EBPB */
 /* 27*/	unsigned char	ms_serno[4];
+/* DOS 4.0 EBPB */
 /* 2b*/	unsigned char	ms_label[11];
 /* 36*/	unsigned char   ms_magic[8];
+/* padding */
 /* 3e*/	unsigned char	ms_dummy2[0x1fe - 0x3e];
 /*1fe*/	unsigned char	ms_pmagic[2];
 } __attribute__((packed));
@@ -119,10 +127,9 @@ static const char *no_name = "NO NAME    ";
 /*
  * Look for LABEL (name) in the FAT root directory.
  */
-static unsigned char *search_fat_label(blkid_probe pr,
-				uint64_t offset, uint32_t entries)
+static int search_fat_label(blkid_probe pr, uint64_t offset, uint32_t entries, unsigned char out[11])
 {
-	struct vfat_dir_entry *ent, *dir = NULL;
+	const struct vfat_dir_entry *ent, *dir = NULL;
 	uint32_t i;
 
 	DBG(LOWPROBE, ul_debug("\tlook for label in root-dir "
@@ -136,7 +143,7 @@ static unsigned char *search_fat_label(blkid_probe pr,
 					(uint64_t) entries *
 						sizeof(struct vfat_dir_entry));
 		if (!dir)
-			return NULL;
+			return 0;
 	}
 
 	for (i = 0; i < entries; i++) {
@@ -165,22 +172,24 @@ static unsigned char *search_fat_label(blkid_probe pr,
 		if ((ent->attr & (FAT_ATTR_VOLUME_ID | FAT_ATTR_DIR)) ==
 		    FAT_ATTR_VOLUME_ID) {
 			DBG(LOWPROBE, ul_debug("\tfound fs LABEL at entry %d", i));
-			if (ent->name[0] == 0x05)
-				ent->name[0] = 0xE5;
-			return ent->name;
+			memcpy(out, ent->name, 11);
+			if (out[0] == 0x05)
+				out[0] = 0xE5;
+			return 1;
 		}
 	}
-	return NULL;
+	return 0;
 }
 
 static int fat_valid_superblock(blkid_probe pr,
 			const struct blkid_idmag *mag,
-			struct msdos_super_block *ms,
-			struct vfat_super_block *vs,
-			uint32_t *cluster_count, uint32_t *fat_size)
+			const struct msdos_super_block *ms,
+			const struct vfat_super_block *vs,
+			uint32_t *cluster_count, uint32_t *fat_size,
+			uint32_t *sect_count)
 {
 	uint16_t sector_size, dir_entries, reserved;
-	uint32_t sect_count, __fat_size, dir_size, __cluster_count, fat_length;
+	uint32_t __sect_count, __fat_size, dir_size, __cluster_count, fat_length;
 	uint32_t max_count;
 
 	/* extra check for FATs without magic strings */
@@ -222,10 +231,10 @@ static int fat_valid_superblock(blkid_probe pr,
 
 	dir_entries = unaligned_le16(&ms->ms_dir_entries);
 	reserved =  le16_to_cpu(ms->ms_reserved);
-	sect_count = unaligned_le16(&ms->ms_sectors);
+	__sect_count = unaligned_le16(&ms->ms_sectors);
 
-	if (sect_count == 0)
-		sect_count = le32_to_cpu(ms->ms_total_sect);
+	if (__sect_count == 0)
+		__sect_count = le32_to_cpu(ms->ms_total_sect);
 
 	fat_length = le16_to_cpu(ms->ms_fat_length);
 	if (fat_length == 0)
@@ -235,7 +244,7 @@ static int fat_valid_superblock(blkid_probe pr,
 	dir_size = ((dir_entries * sizeof(struct vfat_dir_entry)) +
 					(sector_size-1)) / sector_size;
 
-	__cluster_count = (sect_count - (reserved + __fat_size + dir_size)) /
+	__cluster_count = (__sect_count - (reserved + __fat_size + dir_size)) /
 							ms->ms_cluster_size;
 	if (!ms->ms_fat_length && vs->vs_fat32_length)
 		max_count = FAT32_MAX;
@@ -249,24 +258,8 @@ static int fat_valid_superblock(blkid_probe pr,
 		*fat_size = __fat_size;
 	if (cluster_count)
 		*cluster_count = __cluster_count;
-
-	if (blkid_probe_is_wholedisk(pr)) {
-		/* OK, seems like FAT, but it's possible that we found boot
-		 * sector with crazy FAT-like stuff (magic strings, media,
-		 * etc..) before MBR. Let's make sure that there is no MBR with
-		 * usable partition. */
-		unsigned char *buf = (unsigned char *) ms;
-
-		if (mbr_is_valid_magic(buf)) {
-			struct dos_partition *p0 = mbr_get_partition(buf, 0);
-
-			if (dos_partition_get_size(p0) != 0 &&
-			    (p0->boot_ind == 0 || p0->boot_ind == 0x80)) {
-				DBG(LOWPROBE, ul_debug("\tMBR detected"));
-				return 0;
-			}
-		}
-	}
+	if (sect_count)
+		*sect_count = __sect_count;
 
 	if (blkid_probe_is_bitlocker(pr))
 		return 0;
@@ -283,8 +276,8 @@ extern int blkid_probe_is_vfat(blkid_probe pr);
  */
 int blkid_probe_is_vfat(blkid_probe pr)
 {
-	struct vfat_super_block *vs;
-	struct msdos_super_block *ms;
+	const struct vfat_super_block *vs;
+	const struct msdos_super_block *ms;
 	const struct blkid_idmag *mag = NULL;
 	int rc;
 
@@ -301,20 +294,21 @@ int blkid_probe_is_vfat(blkid_probe pr)
 	if (!vs)
 		return errno ? -errno : 0;
 
-	return fat_valid_superblock(pr, mag, ms, vs, NULL, NULL);
+	return fat_valid_superblock(pr, mag, ms, vs, NULL, NULL, NULL);
 }
 
 /* FAT label extraction from the root directory taken from Kay
  * Sievers's volume_id library */
 static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 {
-	struct vfat_super_block *vs;
-	struct msdos_super_block *ms;
+	const struct vfat_super_block *vs;
+	const struct msdos_super_block *ms;
 	const unsigned char *vol_label = NULL;
 	const unsigned char *boot_label = NULL;
-	unsigned char *vol_serno = NULL, vol_label_buf[11];
+	const unsigned char *vol_serno = NULL;
+	unsigned char vol_label_buf[11];
 	uint16_t sector_size = 0, reserved;
-	uint32_t cluster_count, fat_size;
+	uint32_t cluster_count, fat_size, sect_count;
 	const char *version = NULL;
 
 	ms = blkid_probe_get_sb(pr, mag, struct msdos_super_block);
@@ -325,7 +319,8 @@ static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 	if (!vs)
 		return errno ? -errno : 1;
 
-	if (!fat_valid_superblock(pr, mag, ms, vs, &cluster_count, &fat_size))
+	if (!fat_valid_superblock(pr, mag, ms, vs, &cluster_count, &fat_size,
+				&sect_count))
 		return 1;
 
 	sector_size = unaligned_le16(&ms->ms_sector_size);
@@ -336,14 +331,14 @@ static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 		uint32_t root_start = (reserved + fat_size) * sector_size;
 		uint32_t root_dir_entries = unaligned_le16(&vs->vs_dir_entries);
 
-		vol_label = search_fat_label(pr, root_start, root_dir_entries);
-		if (vol_label) {
-			memcpy(vol_label_buf, vol_label, 11);
+		if (search_fat_label(pr, root_start, root_dir_entries, vol_label_buf))
 			vol_label = vol_label_buf;
-		}
 
-		boot_label = ms->ms_label;
-		vol_serno = ms->ms_serno;
+		if (ms->ms_ext_boot_sign == 0x29)
+			boot_label = ms->ms_label;
+
+		if (ms->ms_ext_boot_sign == 0x28 || ms->ms_ext_boot_sign == 0x29)
+			vol_serno = ms->ms_serno;
 
 		blkid_probe_set_value(pr, "SEC_TYPE", (unsigned char *) "msdos",
                               sizeof("msdos"));
@@ -354,7 +349,7 @@ static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 			version = "FAT16";
 
 	} else if (vs->vs_fat32_length) {
-		unsigned char *buf;
+		const unsigned char *buf;
 		uint16_t fsinfo_sect;
 		int maxloop = 100;
 
@@ -376,9 +371,7 @@ static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 
 			count = buf_size / sizeof(struct vfat_dir_entry);
 
-			vol_label = search_fat_label(pr, next_off, count);
-			if (vol_label) {
-				memcpy(vol_label_buf, vol_label, 11);
+			if (search_fat_label(pr, next_off, count, vol_label_buf)) {
 				vol_label = vol_label_buf;
 				break;
 			}
@@ -396,7 +389,9 @@ static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 
 		version = "FAT32";
 
-		boot_label = vs->vs_label;
+		if (vs->vs_ext_boot_sign == 0x29)
+			boot_label = vs->vs_label;
+
 		vol_serno = vs->vs_serno;
 
 		/*
@@ -438,7 +433,9 @@ static int probe_vfat(blkid_probe pr, const struct blkid_idmag *mag)
 	if (version)
 		blkid_probe_set_version(pr, version);
 
+	blkid_probe_set_fsblocksize(pr, vs->vs_cluster_size * sector_size);
 	blkid_probe_set_block_size(pr, sector_size);
+	blkid_probe_set_fssize(pr, (uint64_t) sector_size * sect_count);
 
 	return 0;
 }
