@@ -227,6 +227,8 @@ int fdisk_override_geometry(struct fdisk_context *cxt,
 			    unsigned int heads,
 			    unsigned int sectors)
 {
+	int rc = 0;
+
 	if (!cxt)
 		return -EINVAL;
 	if (heads)
@@ -239,14 +241,15 @@ int fdisk_override_geometry(struct fdisk_context *cxt,
 	else
 		recount_geometry(cxt);
 
-	fdisk_reset_alignment(cxt);
+	rc = fdisk_reset_alignment(cxt);
 
-	DBG(CXT, ul_debugobj(cxt, "override C/H/S: %u/%u/%u",
+	DBG(CXT, ul_debugobj(cxt, "override C/H/S: %u/%u/%u [rc=%d]",
 		(unsigned) cxt->geom.cylinders,
 		(unsigned) cxt->geom.heads,
-		(unsigned) cxt->geom.sectors));
+		(unsigned) cxt->geom.sectors,
+		rc));
 
-	return 0;
+	return rc;
 }
 
 /**
@@ -365,6 +368,8 @@ int fdisk_has_user_device_geometry(struct fdisk_context *cxt)
 
 int fdisk_apply_user_device_properties(struct fdisk_context *cxt)
 {
+	int rc;
+
 	if (!cxt)
 		return -EINVAL;
 
@@ -395,7 +400,9 @@ int fdisk_apply_user_device_properties(struct fdisk_context *cxt)
 	else if (cxt->user_geom.heads || cxt->user_geom.sectors)
 		recount_geometry(cxt);
 
-	fdisk_reset_alignment(cxt);
+	rc = fdisk_reset_alignment(cxt);
+	if (rc)
+		return rc;
 
 	if (cxt->user_grain) {
 		unsigned long granularity = max(cxt->phy_sector_size, cxt->min_io_size);
@@ -467,8 +474,7 @@ int fdisk_reset_device_properties(struct fdisk_context *cxt)
 	if (rc)
 		return rc;
 
-	fdisk_apply_user_device_properties(cxt);
-	return 0;
+	return fdisk_apply_user_device_properties(cxt);
 }
 
 /*
@@ -541,12 +547,16 @@ int fdisk_discover_topology(struct fdisk_context *cxt)
 				/* optimal I/O is optional, default to minimum IO */
 				cxt->io_size = cxt->min_io_size;
 
-			/* ignore optimal I/O if not aligned to phy.sector size */
-			if (cxt->io_size
-			    && cxt->phy_sector_size
-			    && (cxt->io_size % cxt->phy_sector_size) != 0) {
-				DBG(CXT, ul_debugobj(cxt, "ignore misaligned I/O size"));
-				cxt->io_size = cxt->phy_sector_size;
+			if (cxt->io_size && cxt->phy_sector_size) {
+				if (cxt->io_size == 33553920) {
+					/* 33553920 (32 MiB - 512) is always a controller error */
+					DBG(CXT, ul_debugobj(cxt, "ignore bad I/O size 33553920"));
+					cxt->io_size = cxt->phy_sector_size;
+				} else if ((cxt->io_size % cxt->phy_sector_size) != 0) {
+					/* ignore optimal I/O if not aligned to phy.sector size */
+					DBG(CXT, ul_debugobj(cxt, "ignore misaligned I/O size"));
+					cxt->io_size = cxt->phy_sector_size;
+				}
 			}
 
 		}
