@@ -11,6 +11,7 @@
 #define _LIBFDISK_PRIVATE_H
 
 #include <errno.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -300,8 +301,8 @@ struct fdisk_label {
 	struct fdisk_geometry	geom_min;	/* minimal geometry */
 	struct fdisk_geometry	geom_max;	/* maximal geometry */
 
-	unsigned int		changed:1,	/* label has been modified */
-				disabled:1;	/* this driver is disabled at all */
+	bool			changed,	/* label has been modified */
+				disabled;	/* this driver is disabled at all */
 
 	const struct fdisk_field *fields;	/* all possible fields */
 	size_t			nfields;
@@ -351,9 +352,9 @@ struct fdisk_ask {
 			uint64_t	base;		/* for relative results */
 			uint64_t	unit;		/* unit for offsets */
 			const char	*range;		/* by library generated list */
-			unsigned int	relative :1,
-					inchars  :1,
-					wrap_negative	:1;
+			bool		relative,
+					inchars,
+					wrap_negative;
 		} num;
 		/* FDISK_ASKTYPE_{WARN,WARNX,..} */
 		struct ask_print {
@@ -494,7 +495,7 @@ struct fdisk_labelitem {
 };
 
 /* Use only internally for non-allocated items, never use
- * refcouting for such items!
+ * refcounting for such items!
  */
 #define FDISK_LABELITEM_INIT	{ .type = 0, .refcount = 0 }
 
