@@ -35,7 +35,7 @@ struct fdisk_sun_label {
 	struct sun_disklabel   *header;		/* on-disk data (pointer to cxt->firstsector) */
 };
 
-static struct fdisk_parttype sun_parttypes[] = {
+static const struct fdisk_parttype sun_parttypes[] = {
 	{SUN_TAG_UNASSIGNED, N_("Unassigned")},
 	{SUN_TAG_BOOT, N_("Boot")},
 	{SUN_TAG_ROOT, N_("SunOS root")},
@@ -83,10 +83,10 @@ static void set_partition(struct fdisk_context *cxt, size_t i,
 			fdisk_label_get_parttype_from_code(cxt->label, sysid);
 
 	if (start / (cxt->geom.heads * cxt->geom.sectors) > UINT32_MAX)
-		fdisk_warnx(cxt, _("%#zu: start cylinder overflows Sun label limits"), i+1);
+		fdisk_warnx(cxt, _("#%zu: start cylinder overflows Sun label limits"), i+1);
 
 	if (stop - start > UINT32_MAX)
-		fdisk_warnx(cxt, _("%#zu: number of sectors overflow Sun label limits"), i+1);
+		fdisk_warnx(cxt, _("#%zu: number of sectors overflow Sun label limits"), i+1);
 
 	sunlabel->vtoc.infos[i].id = cpu_to_be16(sysid);
 	sunlabel->vtoc.infos[i].flags = cpu_to_be16(0);
@@ -246,10 +246,10 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 		} else {
 			fdisk_warnx(cxt,
 				_("BLKGETSIZE ioctl failed on %s. "
-				  "Using geometry cylinder value of %llu. "
+				  "Using geometry cylinder value of %ju. "
 				  "This value may be truncated for devices "
 				  "> 33.8 GB."),
-				cxt->dev_path, cxt->geom.cylinders);
+				cxt->dev_path, (uintmax_t) cxt->geom.cylinders);
 		}
 	} else
 		ask_geom(cxt);
@@ -383,6 +383,10 @@ static void fetch_sun(struct fdisk_context *cxt,
 			lens[i] = 0;
 		}
 	}
+	for (i = cxt->label->nparts_max; i < SUN_MAXPARTITIONS; i++) {
+		starts[i] = 0;
+		lens[i] = 0;
+	}
 }
 
 /* non-Linux qsort_r(3) has usually differently ordered arguments */
@@ -1186,5 +1190,7 @@ struct fdisk_label *fdisk_new_sun_label(struct fdisk_context *cxt __attribute__
 	lb->geom_max.sectors = 1024;
 	lb->geom_max.heads = 1024;
 	lb->geom_max.cylinders = USHRT_MAX;
-	return lb;
+
+	/* return calloc() result to keep static anaylizers happy */
+	return (struct fdisk_label *) sun;
 }
