@@ -143,7 +143,7 @@ char *fdisk_partname(const char *dev, size_t partno)
 
 	/* devfs kludge - note: fdisk partition names are not supposed
 	   to equal kernel names, so there is no reason to do this */
-	if (endswith(dev, "disc")) {
+	if (ul_endswith(dev, "disc")) {
 		w -= 4;
 		p = "part";
 	}
@@ -183,11 +183,15 @@ done:
 }
 
 #ifdef TEST_PROGRAM
-struct fdisk_label *fdisk_new_dos_label(struct fdisk_context *cxt) { return NULL; }
-struct fdisk_label *fdisk_new_bsd_label(struct fdisk_context *cxt) { return NULL; }
+struct fdisk_label *fdisk_new_dos_label(struct fdisk_context *cxt __attribute__((unused))) { return NULL; }
+struct fdisk_label *fdisk_new_bsd_label(struct fdisk_context *cxt __attribute__((unused))) { return NULL; }
 
-static int test_partnames(struct fdisk_test *ts, int argc, char *argv[])
+static int test_partnames(struct fdisk_test *ts __attribute__((unused)),
+			  int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	size_t i;
 	const char *disk = argv[1];
 
