@@ -1,16 +1,20 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
+ * This file is part of libmount from util-linux project.
+ *
  * Copyright (C) 2016 David Sterba <dsterba@suse.cz>
  * Copyright (C) 2016 Stanislav Brabec <sbrabec@suse.cz>
  *
- * Based on kernel ctree.h, rbtree.h and btrfs-progs.
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * Based on kernel ctree.h, rbtree.h and btrfs-progs.
  */
 #include <dirent.h>
 #include <sys/ioctl.h>
 #include <stdlib.h>
-#include <stdint.h>
 #include <linux/btrfs.h>
 
 #include "mountP.h"
