@@ -1,8 +1,13 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
- * Copyright (C) 2008 Karel Zak <kzak@redhat.com>
+ * This file is part of libmount from util-linux project.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * Copyright (C) 2008-2018 Karel Zak <kzak@redhat.com>
+ *
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  */
 
 /**
@@ -24,14 +29,17 @@ UL_DEBUG_DEFINE_MASKNAMES(libmount) =
 	{ "diff", MNT_DEBUG_DIFF,	"mountinfo changes tracking" },
 	{ "fs", MNT_DEBUG_FS,		"FS abstraction" },
 	{ "help", MNT_DEBUG_HELP,	"this help" },
+	{ "hook", MNT_DEBUG_HOOK,	"hooks functionality" },
 	{ "locks", MNT_DEBUG_LOCKS,	"mtab and utab locking" },
 	{ "loop", MNT_DEBUG_LOOP,	"loop devices routines" },
 	{ "options", MNT_DEBUG_OPTIONS,	"mount options parsing" },
+	{ "optlist", MNT_DEBUG_OPTLIST, "mount options container" },
 	{ "tab", MNT_DEBUG_TAB,		"fstab, mtab, mountinfo routines" },
 	{ "update", MNT_DEBUG_UPDATE,	"mtab, utab updates" },
 	{ "utils", MNT_DEBUG_UTILS,	"misc library utils" },
 	{ "monitor", MNT_DEBUG_MONITOR,	"mount tables monitor" },
 	{ "btrfs", MNT_DEBUG_BTRFS,	"btrfs specific routines" },
+	{ "verity", MNT_DEBUG_VERITY,	"verity specific routines" },
 
 	{ NULL, 0 }
 };
@@ -61,7 +69,7 @@ void mnt_init_debug(int mask)
 		mnt_get_library_version(&ver);
 		mnt_get_library_features(&features);
 
-		DBG(INIT, ul_debug("library debug mask: 0x%04x", libmount_debug_mask));
+		DBG(INIT, ul_debug("library debug mask: 0x%06x", libmount_debug_mask));
 		DBG(INIT, ul_debug("library version: %s", ver));
 		p = features;
 		while (p && *p)
