@@ -23,13 +23,18 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <stdint.h>
+#include <inttypes.h>
 
 #include "c.h"
+
 #include "list.h"
 #include "debug.h"
 #include "buffer.h"
 #include "libmount.h"
 
+#include "mount-api-utils.h"
+
 /*
  * Debug
  */
@@ -50,6 +55,7 @@
 #define MNT_DEBUG_VERITY	(1 << 14)
 #define MNT_DEBUG_HOOK		(1 << 15)
 #define MNT_DEBUG_OPTLIST	(1 << 16)
+#define MNT_DEBUG_STATMNT	(1 << 17)
 
 #define MNT_DEBUG_ALL		0xFFFFFF
 
@@ -96,6 +102,9 @@ struct libmnt_test {
 extern int mnt_run_test(struct libmnt_test *tests, int argc, char *argv[]);
 #endif
 
+/* private tab_listmount.c */
+struct libmnt_listmnt;
+
 /* utils.c */
 extern int mnt_valid_tagname(const char *tagname);
 
@@ -134,6 +143,8 @@ extern int mnt_is_path(const char *target);
 extern int mnt_tmptgt_unshare(int *old_ns_fd);
 extern int mnt_tmptgt_cleanup(int old_ns_fd);
 
+extern int mnt_id_from_fd(int fd, uint64_t *uniq_id, int *id);
+
 /* tab.c */
 extern int is_mountinfo(struct libmnt_table *tb);
 extern int mnt_table_set_parser_fltrcb(	struct libmnt_table *tb,
@@ -156,6 +167,11 @@ extern int __mnt_table_is_fs_mounted(	struct libmnt_table *tb,
 extern int mnt_table_enable_noautofs(struct libmnt_table *tb, int ignore);
 extern int mnt_table_is_noautofs(struct libmnt_table *tb);
 
+/* tab_listmount.c */
+extern int mnt_table_next_lsmnt(struct libmnt_table *tb, int direction);
+extern int mnt_table_reset_listmount(struct libmnt_table *tb);
+extern int mnt_table_want_listmount(struct libmnt_table *tb);
+
 /*
  * Generic iterator
  */
@@ -185,6 +201,20 @@ struct libmnt_iter {
 	} while(0)
 
 
+/*
+ * statmount setting; shared between tables and filesystems
+ */
+struct libmnt_statmnt {
+	int             refcount;
+	uint64_t        mask;           /* default statmount() mask */
+
+	struct ul_statmount *buf;
+	size_t bufsiz;
+
+	unsigned int    disabled: 1;    /* enable or disable statmount() */
+};
+
+
 /*
  * This struct represents one entry in a fstab/mountinfo file.
  * (note that fstab[1] means the first column from fstab, and so on...)
@@ -199,7 +229,11 @@ struct libmnt_fs {
 	struct libmnt_optlist *optlist;
 
 	int		id;		/* mountinfo[1]: ID */
+	uint64_t	uniq_id;	/* unique node ID; statx(STATX_MNT_ID_UNIQUE); statmount->mnt_id */
+	uint64_t        ns_id;		/* namespace ID; statmount->mnt_ns_id */
+
 	int		parent;		/* mountinfo[2]: parent */
+	uint64_t	uniq_parent;	/* unique parent ID; statmount->mnt_parent_id */
 	dev_t		devno;		/* mountinfo[3]: st_dev */
 
 	char		*bindsrc;	/* utab, full path from fstab[1] for bind mounts */
@@ -215,7 +249,10 @@ struct libmnt_fs {
 
 	char		*optstr;	/* fstab[4], merged options */
 	char		*vfs_optstr;	/* mountinfo[6]: fs-independent (VFS) options */
+
 	char		*opt_fields;	/* mountinfo[7]: optional fields */
+	uint64_t	propagation;	/* statmmount() or parsed opt_fields */
+
 	char		*fs_optstr;	/* mountinfo[11]: fs-dependent options */
 	char		*user_optstr;	/* userspace mount options */
 	char		*attrs;		/* mount attributes */
@@ -232,6 +269,9 @@ struct libmnt_fs {
 	int		flags;		/* MNT_FS_* flags */
 	pid_t		tid;		/* /proc/<tid>/mountinfo otherwise zero */
 
+	uint64_t	stmnt_done;	/* mask of already called masks */
+	struct libmnt_statmnt *stmnt;	/* statmount() stuff */
+
 	char		*comment;	/* fstab comment */
 
 	void		*userdata;	/* library independent data */
@@ -246,6 +286,16 @@ struct libmnt_fs {
 #define MNT_FS_KERNEL	(1 << 4) /* data from /proc/{mounts,self/mountinfo} */
 #define MNT_FS_MERGED	(1 << 5) /* already merged data from /run/mount/utab */
 
+#ifdef HAVE_STATMOUNT_API
+# define	mnt_fs_try_statmount(FS, MEMBER, FLAGS) __extension__ ({	\
+			if (!(FS)->MEMBER					\
+			    && (FS)->stmnt					\
+			    && !(FS)->stmnt->disabled				\
+			    && ((FLAGS) & ~((FS)->stmnt_done)))			\
+				mnt_fs_fetch_statmount((FS), (FLAGS)); })
+#endif
+
+
 /*
  * fstab/mountinfo file
  */
@@ -265,6 +315,9 @@ struct libmnt_table {
 	int		(*fltrcb)(struct libmnt_fs *fs, void *data);
 	void		*fltrcb_data;
 
+	struct libmnt_listmnt	*lsmnt;	/* listmount() stuff */
+	struct libmnt_statmnt	*stmnt; /* statmount() stuff */
+
 	int		noautofs;	/* ignore autofs mounts */
 
 	struct list_head	ents;	/* list of entries (libmnt_fs) */
@@ -413,7 +466,7 @@ struct libmnt_context
 	const struct libmnt_optmap *map_linux;		/* system options map */
 	const struct libmnt_optmap *map_userspace;	/* userspace options map */
 
-	const char	*mountinfo_path; /* usualy /proc/self/moutinfo */
+	const char	*mountinfo_path; /* usually /proc/self/moutinfo */
 
 	const char	*utab_path; /* path to utab */
 	int		utab_writable; /* is utab writable */
@@ -432,7 +485,8 @@ struct libmnt_context
 
 	int	syscall_status;	/* 1: not called yet, 0: success, <0: -errno */
 	const char *syscall_name;	/* failed syscall name */
-	char	*syscall_errmsg;	/* message from kernel */
+
+	char	**mesgs;		/* library or kernel messages (NULL terminated array) */
 
 	struct libmnt_ns	ns_orig;	/* original namespace */
 	struct libmnt_ns	ns_tgt;		/* target namespace */
@@ -480,27 +534,6 @@ struct libmnt_context
 /* Flags usable with MS_BIND|MS_REMOUNT */
 #define MNT_BIND_SETTABLE	(MS_NOSUID|MS_NODEV|MS_NOEXEC|MS_NOATIME|MS_NODIRATIME|MS_RELATIME|MS_RDONLY|MS_NOSYMFOLLOW)
 
-static inline void set_syscall_status(struct libmnt_context *cxt, const char *name, int x)
-{
-	if (!x) {
-		DBG(CXT, ul_debug("syscall '%s' [%m]", name));
-		cxt->syscall_status = -errno;
-		cxt->syscall_name = name;
-	} else {
-		DBG(CXT, ul_debug("syscall '%s' [success]", name));
-		cxt->syscall_status = 0;
-	}
-}
-
-static inline void reset_syscall_status(struct libmnt_context *cxt)
-{
-	DBG(CXT, ul_debug("reset syscall status"));
-	cxt->syscall_status = 0;
-	cxt->syscall_name = NULL;
-
-	free(cxt->syscall_errmsg);
-	cxt->syscall_errmsg = NULL;
-}
 
 /* optmap.c */
 extern const struct libmnt_optmap *mnt_optmap_get_entry(
@@ -589,7 +622,7 @@ extern int mnt_optlist_strdup_optstr(struct libmnt_optlist *ls, char **optstr,
 extern int mnt_optlist_get_propagation(struct libmnt_optlist *ls);
 extern int mnt_optlist_is_propagation_only(struct libmnt_optlist *ls);
 extern int mnt_optlist_is_remount(struct libmnt_optlist *ls);
-extern int mnt_optlist_is_recursive(struct libmnt_optlist *ls);
+extern int mnt_optlist_is_rpropagation(struct libmnt_optlist *ls);
 extern int mnt_optlist_is_bind(struct libmnt_optlist *ls);
 extern int mnt_optlist_is_rbind(struct libmnt_optlist *ls);
 extern int mnt_optlist_is_move(struct libmnt_optlist *ls);
@@ -609,6 +642,7 @@ extern int mnt_opt_set_u64value(struct libmnt_opt *opt, uint64_t num);
 extern int mnt_opt_set_quoted_value(struct libmnt_opt *opt, const char *str);
 extern int mnt_opt_is_external(struct libmnt_opt *opt);
 extern int mnt_opt_is_sepnodata(struct libmnt_opt *opt);
+extern int mnt_opt_value_with(struct libmnt_opt *opt, const char *str);
 
 /* fs.c */
 extern int mnt_fs_follow_optlist(struct libmnt_fs *fs, struct libmnt_optlist *ol);
@@ -621,6 +655,10 @@ extern int __mnt_fs_set_target_ptr(struct libmnt_fs *fs, char *tgt)
 			__attribute__((nonnull(1)));
 
 /* context.c */
+extern void mnt_context_syscall_save_status(struct libmnt_context *cxt,
+                                        const char *syscallname, int success);
+extern void mnt_context_syscall_reset_status(struct libmnt_context *cxt);
+
 extern struct libmnt_context *mnt_copy_context(struct libmnt_context *o);
 extern int mnt_context_utab_writable(struct libmnt_context *cxt);
 extern const char *mnt_context_get_writable_tabpath(struct libmnt_context *cxt);
@@ -643,6 +681,11 @@ extern int mnt_context_update_tabs(struct libmnt_context *cxt);
 extern int mnt_context_umount_setopt(struct libmnt_context *cxt, int c, char *arg);
 extern int mnt_context_mount_setopt(struct libmnt_context *cxt, int c, char *arg);
 
+extern void mnt_context_reset_mesgs(struct libmnt_context *cxt);
+extern int mnt_context_append_mesg(struct libmnt_context *cxt, const char *msg);
+extern int mnt_context_sprintf_mesg(struct libmnt_context *cxt, const char *msg, ...);
+extern int mnt_context_read_mesgs(struct libmnt_context *cxt, int fd);
+
 extern int mnt_context_propagation_only(struct libmnt_context *cxt)
 			__attribute__((nonnull));
 
@@ -667,6 +710,8 @@ extern int mnt_context_apply_fs(struct libmnt_context *cxt, struct libmnt_fs *fs
 
 extern struct libmnt_optlist *mnt_context_get_optlist(struct libmnt_context *cxt);
 
+extern int mnt_context_is_xnocanonicalize(struct libmnt_context *cxt, const char *type);
+
 /* tab_update.c */
 extern int mnt_update_emit_event(struct libmnt_update *upd);
 extern int mnt_update_set_filename(struct libmnt_update *upd, const char *filename);
