@@ -16,7 +16,6 @@
 #include "strutils.h"
 #include "list.h"
 #include "mountP.h"
-#include "mount-api-utils.h"
 
 #define MNT_OL_MAXMAPS	8
 
@@ -72,7 +71,7 @@ struct libmnt_optlist {
 			is_rdonly : 1,
 			is_move : 1,
 			is_silent : 1,
-			is_recursive : 1;
+			is_rpropagation : 1;	/* recursive propagation (rbind, rprivate, ...) */
 };
 
 struct libmnt_optlist *mnt_new_optlist(void)
@@ -139,7 +138,7 @@ int mnt_optlist_register_map(struct libmnt_optlist *ls, const struct libmnt_optm
 
 	for (i = 0; i < ls->nmaps; i++) {
 		if (ls->maps[i] == map)
-			return 0;		/* already registred, ignore */
+			return 0;		/* already registered, ignore */
 	}
 	if (ls->nmaps + 1 >= MNT_OL_MAXMAPS)
 		return -ERANGE;
@@ -203,7 +202,7 @@ int mnt_optlist_remove_opt(struct libmnt_optlist *ls, struct libmnt_opt *opt)
 			ls->is_silent = 0;
 
 		if (opt->ent->id & MS_REC)
-			ls->is_recursive = 0;
+			ls->is_rpropagation = 0;
 	}
 
 	optlist_cleanup_cache(ls);
@@ -475,13 +474,13 @@ static struct libmnt_opt *optlist_new_opt(struct libmnt_optlist *ls,
 			ls->is_silent = 1;
 
 		if (opt->ent->id & MS_REC) {
-			ls->is_recursive = 1;
+			ls->is_rpropagation = 1;
 			opt->recursive = 1;
 		}
 	}
 #ifdef USE_LIBMOUNT_MOUNTFD_SUPPORT
 	if (!opt->recursive && opt->value
-	    && is_vfs_opt(opt) && strcmp(opt->value, "recursive") == 0)
+	    && is_vfs_opt(opt) && mnt_opt_value_with(opt, "recursive"))
 		opt->recursive = 1;
 #endif
 	if (ent && map) {
@@ -837,6 +836,7 @@ int mnt_optlist_get_attrs(struct libmnt_optlist *ls, uint64_t *set, uint64_t *cl
 	struct libmnt_iter itr;
 	struct libmnt_opt *opt;
 	uint64_t remount_reset = 0;
+	uint64_t atime_set = 0;
 
 	if (!ls || !ls->linux_map || !set || !clr)
 		return -EINVAL;
@@ -848,7 +848,7 @@ int mnt_optlist_get_attrs(struct libmnt_optlist *ls, uint64_t *set, uint64_t *cl
 	 * specified (except atime stuff). For backward compatibility we need
 	 * to emulate this semantic by mount_setattr(). The new
 	 * mount_setattr() has simple set/unset sematinc and nothing is
-	 * internally in kernel reseted.
+	 * reset internally in kernel.
 	 */
 	if (mnt_optlist_is_remount(ls)
 	    && !mnt_optlist_is_bind(ls)
@@ -875,33 +875,45 @@ int mnt_optlist_get_attrs(struct libmnt_optlist *ls, uint64_t *set, uint64_t *cl
 		if (flag_to_attr( opt->ent->id, &x) < 0)
 			continue;
 
+		if (x == MOUNT_ATTR_RDONLY && !mnt_opt_value_with(opt, "vfs")
+				           && mnt_opt_value_with(opt, "fs"))
+			continue;
+
 		if (x && remount_reset)
 			remount_reset &= ~x;
 
 		if (opt->ent->mask & MNT_INVERT) {
-			DBG(OPTLIST, ul_debugobj(ls, " clr: %s", opt->ent->name));
-			/*
-			 * All atime settings are mutually exclusive so *clr must
-			 * have MOUNT_ATTR__ATIME set.
-			 *
-			 * See the function fs/namespace.c:build_mount_kattr()
-			 * in the linux kernel source.
-			 */
+			DBG(OPTLIST, ul_debugobj(ls, " clr: %s 0x%08" PRIx64,
+						opt->ent->name, x));
+
 			if (x == MOUNT_ATTR_RELATIME || x == MOUNT_ATTR_NOATIME ||
 			    x == MOUNT_ATTR_STRICTATIME)
 				*clr |= MOUNT_ATTR__ATIME;
 			else
 				*clr |= x;
 		} else {
-			DBG(OPTLIST, ul_debugobj(ls, " set: %s", opt->ent->name));
-			*set |= x;
-
 			if (x == MOUNT_ATTR_RELATIME || x == MOUNT_ATTR_NOATIME ||
-			    x == MOUNT_ATTR_STRICTATIME)
+			    x == MOUNT_ATTR_STRICTATIME) {
+				/* All atime settings are mutually exclusive,
+				 * the last option wins and MOUNT_ATTR__ATIME
+				 * is required in clr mask.
+				 */
+				DBG(OPTLIST, ul_debugobj(ls, " atime: %s 0x%08" PRIx64,
+							opt->ent->name, x));
 				*clr |= MOUNT_ATTR__ATIME;
+				atime_set = x;
+			} else {
+				DBG(OPTLIST, ul_debugobj(ls, " set: %s 0x%08" PRIx64,
+							opt->ent->name, x));
+				*set |= x;
+			}
 		}
 	}
 
+	if (atime_set) {
+		DBG(OPTLIST, ul_debugobj(ls, " set atime 0x%08" PRIx64, atime_set));
+		*set |= atime_set;
+	}
 	if (remount_reset)
 		*clr |= remount_reset;
 
@@ -937,7 +949,7 @@ int mnt_optlist_strdup_optstr(struct libmnt_optlist *ls, char **optstr,
 
 	*optstr = NULL;
 
-	/* For generic options srings ro/rw is expected at the begining */
+	/* For generic options strings ro/rw are expected at the beginning */
 	if ((!map || map == ls->linux_map)
 	     && (what == MNT_OL_FLTR_DFLT ||
 		 what == MNT_OL_FLTR_ALL ||
@@ -955,6 +967,17 @@ int mnt_optlist_strdup_optstr(struct libmnt_optlist *ls, char **optstr,
 		if (!opt->name)
 			continue;
 		if (opt->map == ls->linux_map && opt->ent->id == MS_RDONLY) {
+			/* "ro=fs" -- ignore for VFS (linux map wanted) */
+			if (map == ls->linux_map
+			    && !mnt_opt_value_with(opt, "vfs")
+			    && mnt_opt_value_with(opt, "fs"))
+				continue;
+			/* "ro=vfs" -- ignore for FS */
+			if (what == MNT_OL_FLTR_UNKNOWN
+			    && !mnt_opt_value_with(opt, "fs")
+			    && mnt_opt_value_with(opt, "vfs"))
+				continue;
+
 			is_rdonly = opt->ent->mask & MNT_INVERT ? 0 : 1;
 			continue;
 		}
@@ -1095,9 +1118,9 @@ int mnt_optlist_is_remount(struct libmnt_optlist *ls)
 	return ls && ls->is_remount;
 }
 
-int mnt_optlist_is_recursive(struct libmnt_optlist *ls)
+int mnt_optlist_is_rpropagation(struct libmnt_optlist *ls)
 {
-	return ls && ls->is_recursive;
+	return ls && ls->is_rpropagation;
 }
 
 int mnt_optlist_is_move(struct libmnt_optlist *ls)
@@ -1136,6 +1159,34 @@ const char *mnt_opt_get_value(struct libmnt_opt *opt)
 	return opt->value;
 }
 
+/* check if option value is @str or comma separated @str */
+int mnt_opt_value_with(struct libmnt_opt *opt, const char *str)
+{
+	char *p;
+	const char *start = opt->value;
+	size_t len;
+
+	if (!str || !opt->value || !*opt->value)
+		return 0;
+
+	len = strlen(str);
+
+	while (start && *start) {
+		p = strstr(start, str);
+		if (!p)
+			return 0;
+		start = p + len;
+
+		if (p > opt->value && *(p - 1) != ',')
+			continue;
+		len = strlen(str);
+		if (*(p + len) != '\0' && *(p + len) != ',')
+			continue;
+		return 1;
+	}
+	return 0;
+}
+
 const char *mnt_opt_get_name(struct libmnt_opt *opt)
 {
 	return opt->name;
@@ -1158,7 +1209,7 @@ int mnt_opt_set_value(struct libmnt_opt *opt, const char *str)
 	opt->recursive = 0;
 	rc = strdup_to_struct_member(opt, value, str);
 
-	if (rc == 0 && str && strcmp(str, "recursive") == 0)
+	if (rc == 0 && mnt_opt_value_with(opt, "recursive"))
 		opt->recursive = 1;
 	return rc;
 }
@@ -1250,10 +1301,11 @@ static const struct libmnt_optmap *get_map(const char *name)
 
 static inline unsigned long str2flg(const char *str)
 {
-	return (unsigned long) strtox64_or_err(str, "connt convert string to flags");
+	return (unsigned long) strtox64_or_err(str, "can't convert string to flags");
 }
 
-static int test_append_str(struct libmnt_test *ts, int argc, char *argv[])
+static int test_append_str(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	int rc;
@@ -1269,7 +1321,8 @@ static int test_append_str(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_prepend_str(struct libmnt_test *ts, int argc, char *argv[])
+static int test_prepend_str(struct libmnt_test *ts __attribute__((unused)),
+			    int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	int rc;
@@ -1285,7 +1338,8 @@ static int test_prepend_str(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_set_str(struct libmnt_test *ts, int argc, char *argv[])
+static int test_set_str(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	int rc;
@@ -1301,7 +1355,8 @@ static int test_set_str(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_append_flg(struct libmnt_test *ts, int argc, char *argv[])
+static int test_append_flg(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	int rc;
@@ -1317,7 +1372,8 @@ static int test_append_flg(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_set_flg(struct libmnt_test *ts, int argc, char *argv[])
+static int test_set_flg(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	int rc;
@@ -1333,7 +1389,8 @@ static int test_set_flg(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_get_str(struct libmnt_test *ts, int argc, char *argv[])
+static int test_get_str(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	const struct libmnt_optmap *map;
@@ -1392,7 +1449,8 @@ done:
 	return rc;
 }
 
-static int test_get_flg(struct libmnt_test *ts, int argc, char *argv[])
+static int test_get_flg(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	struct libmnt_optlist *ol;
 	unsigned long flags = 0;
@@ -1409,6 +1467,62 @@ static int test_get_flg(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
+static int test_split(struct libmnt_test *ts __attribute__((unused)),
+		      int argc, char *argv[])
+{
+	struct libmnt_optlist *ol;
+	int rc;
+	struct libmnt_iter itr;
+	struct libmnt_opt *opt;
+	const char *name, *value;
+
+	if (argc != 2)
+		return -EINVAL;
+	rc = mk_optlist(&ol, argv[1]);
+	if (rc)
+		goto done;
+
+	mnt_reset_iter(&itr, MNT_ITER_FORWARD);
+
+	while (mnt_optlist_next_opt(ol, &itr, &opt) == 0) {
+		name = mnt_opt_get_name(opt);
+		value = mnt_opt_get_value(opt);
+
+		printf("%s = %s\n", name, value ?: "(null)");
+	}
+
+done:
+	mnt_unref_optlist(ol);
+	return rc;
+}
+
+static int test_value_with(struct libmnt_test *ts __attribute__((unused)),
+		int argc, char *argv[])
+{
+	struct libmnt_optlist *ol;
+	struct libmnt_opt *opt;
+	int rc;
+
+	if (argc != 4)
+		return -EINVAL;
+
+	rc = mk_optlist(&ol, argv[1]);
+	if (rc)
+		goto done;
+
+	opt = mnt_optlist_get_named(ol, argv[2], NULL);
+	if (!opt)
+		warnx("not found '%s' option", argv[2]);
+	else if (mnt_opt_value_with(opt, argv[3]))
+		printf("found\n");
+	else
+		printf("'%s' not found within '%s'\n", argv[3], mnt_opt_get_value(opt));
+
+done:
+	mnt_unref_optlist(ol);
+	return rc;
+}
+
 int main(int argc, char *argv[])
 {
 	struct libmnt_test tss[] = {
@@ -1419,6 +1533,8 @@ int main(int argc, char *argv[])
 		{ "--set-flg",     test_set_flg,     "<list> <flg>  linux|user   set to the list" },
 		{ "--get-str",     test_get_str,     "<list> [linux|user]        all options in string" },
 		{ "--get-flg",     test_get_flg,     "<list>  linux|user         all options by flags" },
+		{ "--split",       test_split,       "<list>                     split options into key-value pairs"},
+		{ "--value-with",  test_value_with,  "<list> <opt> <str>        check if option value contains string"},
 
 		{ NULL }
 	};
