@@ -25,8 +25,11 @@ UL_DEBUG_DEFINE_MASKNAMES(libsmartcols) =
 	{ "cell", SCOLS_DEBUG_CELL,	"table cell utils" },
 	{ "col", SCOLS_DEBUG_COL,	"cols utils" },
 	{ "help", SCOLS_DEBUG_HELP,	"this help" },
+	{ "group", SCOLS_DEBUG_GROUP,	"lines grouping utils" },
 	{ "line", SCOLS_DEBUG_LINE,	"table line utils" },
 	{ "tab", SCOLS_DEBUG_TAB,	"table utils" },
+	{ "filter", SCOLS_DEBUG_FLTR,	"lines filter" },
+	{ "fparam", SCOLS_DEBUG_FPARAM, "filter params" },
 	{ NULL, 0, NULL }
 };
 
