@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2009 Karel Zak <kzak@redhat.com>
  *
- * This file may be redistributed under the terms of the GNU Public
- * License.
+ * This file may be redistributed under the terms of the
+ * GNU General Public License.
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,8 +32,8 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_("Find a filesystem by label or UUID.\n"), out);
 
 	fputs(USAGE_OPTIONS, out);
-	printf(USAGE_HELP_OPTIONS(16));
-	printf(USAGE_MAN_TAIL("findfs(8)"));
+	fprintf(out, USAGE_HELP_OPTIONS(16));
+	fprintf(out, USAGE_MAN_TAIL("findfs(8)"));
 	exit(FINDFS_SUCCESS);
 }
 
@@ -76,4 +76,3 @@ int main(int argc, char **argv)
 	puts(dev);
 	return FINDFS_SUCCESS;
 }
-
