@@ -35,12 +35,16 @@ enum {
 	FL_JSON		= (1 << 30),
 };
 
-extern struct libmnt_cache *cache;
-extern unsigned int flags;
-extern int parse_nerrors;
+struct findmnt {
+	struct libmnt_cache *cache;
+	unsigned int flags;
+	int parse_nerrors;
+	char *uri;
+	struct libscols_filter *filter;
+};
 
-extern int is_listall_mode(void);
-extern struct libmnt_fs *get_next_fs(struct libmnt_table *tb, struct libmnt_iter *itr);
-extern int verify_table(struct libmnt_table *tb);
+extern int is_listall_mode(unsigned int flags);
+extern struct libmnt_fs *get_next_fs(struct libmnt_table *tb, struct libmnt_iter *itr, struct findmnt *findmnt);
+extern int verify_table(struct libmnt_table *tb, struct findmnt *findmnt);
 
 #endif /* UTIL_LINUX_FINDMNT_H */
