@@ -57,6 +57,7 @@
 #define CLOSE_EXIT_CODE		XALLOC_EXIT_CODE
 #define TEST_EXIT_CODE		4
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -88,11 +89,10 @@ struct getopt_control {
 	struct option *long_options;	/* long options */
 	int long_options_length;	/* length of options array */
 	int long_options_nr;		/* number of used elements in array */
-	unsigned int
-		compatible:1,		/* compatibility mode for 'difficult' programs */
-		quiet_errors:1,		/* print errors */
-		quiet_output:1,		/* print output */
-		quote:1;		/* quote output */
+	bool	compatible,		/* compatibility mode for 'difficult' programs */
+		quiet_errors,		/* print errors */
+		quiet_output,		/* print output */
+		quote;			/* quote output */
 };
 
 enum { REALLOC_INCREMENT = 8 };
@@ -202,6 +202,10 @@ static int generate_output(struct getopt_control *ctl, char *argv[], int argc)
 		 (argc, argv, ctl->optstr,
 		  (const struct option *)ctl->long_options, &longindex)))
 	       != EOF) {
+		/* Given that these two characters are returned by the getopt(3) routines 
+		 * to distinguish between two distinct internal error states, they should
+		 * not be used as option characters.
+		*/
 		if (opt == '?' || opt == ':')
 			exit_code = GETOPT_EXIT_CODE;
 		else if (!ctl->quiet_output) {
@@ -280,7 +284,7 @@ static void add_short_options(struct getopt_control *ctl, char *options)
 {
 	free(ctl->optstr);
 	if (*options != '+' && getenv("POSIXLY_CORRECT"))
-		ctl->optstr = strconcat("+", options);
+		ctl->optstr = ul_strconcat("+", options);
 	else
 		ctl->optstr = xstrdup(options);
 	if (!ctl->optstr)
