@@ -622,7 +622,7 @@ static void add_structured_data_param(struct list_head *ls, const char *param)
 
 	sd = list_last_entry(ls, struct structured_data, sds);
 
-	if (strv_extend(&sd->params,  param))
+	if (ul_strv_extend(&sd->params,  param))
 		err_oom();
 }
 
@@ -638,7 +638,7 @@ static void __attribute__ ((__format__ (__printf__, 2, 3)))
 
 	sd = list_last_entry(ls, struct structured_data, sds);
 	va_start(ap, fmt);
-	x = strv_extendv(&sd->params, fmt, ap);
+	x = ul_strv_extendv(&sd->params, fmt, ap);
 	va_end(ap);
 
 	if (x)
@@ -649,11 +649,11 @@ static char *strdup_structured_data(struct structured_data *sd)
 {
 	char *res, *tmp;
 
-	if (strv_isempty(sd->params))
+	if (ul_strv_isempty(sd->params))
 		return NULL;
 
 	xasprintf(&res, "[%s %s]", sd->id,
-			(tmp = strv_join(sd->params, " ")));
+			(tmp = ul_strv_join(sd->params, " ")));
 	free(tmp);
 	return res;
 }
@@ -670,7 +670,7 @@ static char *strdup_structured_data_list(struct list_head *ls)
 
 		if (!one)
 			continue;
-		res = strconcat(tmp, one);
+		res = ul_strconcat(tmp, one);
 		free(tmp);
 		free(one);
 	}
@@ -688,7 +688,7 @@ static char *get_structured_data_string(struct logger_ctl *ctl)
 		usr = strdup_structured_data_list(&ctl->user_sds);
 
 	if (sys && usr) {
-		res = strconcat(sys, usr);
+		res = ul_strconcat(sys, usr);
 		free(sys);
 		free(usr);
 	} else
@@ -1025,7 +1025,9 @@ static void logger_stdin(struct logger_ctl *ctl)
 		if (ctl->prio_prefix && c == '<') {
 			pri = 0;
 			buf[i++] = c;
-			while (isdigit(c = getchar()) && pri <= 191) {
+			while (i < ctl->max_message_size
+			       && isdigit(c = getchar()) && pri <= 191) {
+
 				buf[i++] = c;
 				pri = pri * 10 + c - '0';
 			}
@@ -1296,7 +1298,7 @@ int main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 	if (stdout_reopened && argc)
-		warnx(_("--file <file> and <message> are mutually exclusive, message is ignored"));
+		warnx(_("--file <file> and <message> are mutually exclusive; file is ignored"));
 #ifdef HAVE_LIBSYSTEMD
 	if (jfd) {
 		int ret = journald_entry(&ctl, jfd);
