@@ -9,6 +9,7 @@
 #ifndef HWCLOCK_CLOCK_H
 #define HWCLOCK_CLOCK_H
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,35 +36,37 @@ struct hwclock_control {
 #if defined(__linux__) && defined(__alpha__)
 	char *epoch_option;
 #endif
-#ifdef __linux__
+#if defined(__linux__) || defined(__GNU__)
 	char *rtc_dev_name;
+#endif
+#ifdef __linux__
+	uint32_t param_idx;	/* --param-index <n> */
 #endif
 	char *param_get_option;
 	char *param_set_option;
-	unsigned int
-		hwaudit_on:1,
-		adjust:1,
-		show:1,
-		hctosys:1,
-		utc:1,
-		systohc:1,
+	bool	hwaudit_on,
+		adjust,
+		show,
+		hctosys,
+		utc,
+		systohc,
 #if defined(__linux__) && defined(__alpha__)
-		getepoch:1,
-		setepoch:1,
+		getepoch,
+		setepoch,
 #endif
-		noadjfile:1,
-		local_opt:1,
-		directisa:1,
-		testing:1,
-		systz:1,
-		predict:1,
-		get:1,
-		set:1,
-		update:1,
-		universal:1,	/* will store hw_clock_is_utc() return value */
-		vl_read:1,
-		vl_clear:1,
-		verbose:1;
+		noadjfile,
+		local_opt,
+		directisa,
+		testing,
+		systz,
+		predict,
+		get,
+		set,
+		update,
+		universal,	/* will store hw_clock_is_utc() return value */
+		vl_read,
+		vl_clear,
+		verbose;
 };
 
 struct clock_ops {
@@ -78,9 +81,6 @@ struct clock_ops {
 extern const struct clock_ops *probe_for_cmos_clock(void);
 extern const struct clock_ops *probe_for_rtc_clock(const struct hwclock_control *ctl);
 
-/* hwclock.c */
-extern double time_diff(struct timeval subtrahend, struct timeval subtractor);
-
 /* rtc.c */
 #if defined(__linux__) && defined(__alpha__)
 extern int get_epoch_rtc(const struct hwclock_control *ctl, unsigned long *epoch);
