@@ -99,8 +99,12 @@ static const struct id_part arm_part[] = {
     { 0xd87, "Cortex-A725" },
     { 0xd88, "Cortex-A520AE" },
     { 0xd89, "Cortex-A720AE" },
+    { 0xd8a, "C1-Nano" },
+    { 0xd8b, "C1-Pro" },
+    { 0xd8c, "C1-Ultra" },
     { 0xd8e, "Neoverse-N3" },
     { 0xd8f, "Cortex-A320" },
+    { 0xd90, "C1-Premium" },
     { -1, "unknown" },
 };
 
@@ -171,6 +175,7 @@ static const struct id_part nvidia_part[] = {
     { 0x000, "Denver" },
     { 0x003, "Denver 2" },
     { 0x004, "Carmel" },
+    { 0x010, "Olympus" },
     { -1, "unknown" },
 };
 
