@@ -1,4 +1,6 @@
 /*
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
  * mount(8) -- mount a filesystem
  *
  * Copyright (C) 2011 Red Hat, Inc. All rights reserved.
@@ -8,17 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -39,6 +31,7 @@
 #include "closestream.h"
 #include "canonicalize.h"
 #include "pathnames.h"
+#include "strv.h"
 
 #define XALLOC_EXIT_CODE MNT_EX_SYSERR
 #include "xalloc.h"
@@ -66,7 +59,7 @@ static void suid_drop(struct libmnt_context *cxt)
 
 	/* restore "bad" environment variables */
 	if (envs_removed) {
-		env_list_setenv(envs_removed);
+		env_list_setenv(envs_removed, 0);
 		env_list_free(envs_removed);
 		envs_removed = NULL;
 	}
@@ -363,6 +356,51 @@ static void systemd_hint(void)
 # define systemd_hint()
 #endif
 
+static size_t libmount_mesgs(struct libmnt_context *cxt, char type)
+{
+	size_t n = mnt_context_get_nmesgs(cxt, type);
+	char **mesgs = mnt_context_get_mesgs(cxt);
+	char **s;
+
+	if (!n)
+		return 0;
+
+	/* Header */
+	switch (type) {
+		case 'e':
+			fputs(P_("mount error:\n", "mount errors:\n", n), stderr);
+			break;
+		case 'w':
+			fputs(P_("mount warning:\n", "mount warnings:\n", n), stdout);
+			break;
+		case 'i':
+			fputs(P_("mount info:\n", "mount infos:\n", n), stdout);
+			break;
+	}
+
+	/* Messages */
+	UL_STRV_FOREACH(s, mesgs) {
+		switch (type) {
+		case 'e':
+			if (!ul_startswith(*s, "e "))
+				break;
+			fprintf(stderr, "      * %s\n", (*s) + 2);
+			break;
+		case 'w':
+			if (!ul_startswith(*s, "w "))
+				break;
+			fprintf(stdout, "      * %s\n", (*s) + 2);
+			break;
+		case 'i':
+			if (!ul_startswith(*s, "i "))
+				break;
+			fprintf(stdout, "      * %s\n", (*s) + 2);
+			break;
+		}
+	}
+
+	return n;
+}
 
 /*
  * Returns exit status (MNT_EX_*) and/or prints error message.
@@ -375,6 +413,12 @@ static int mk_exit_code(struct libmnt_context *cxt, int rc)
 	rc = mnt_context_get_excode(cxt, rc, buf, sizeof(buf));
 	tgt = mnt_context_get_target(cxt);
 
+	/* error messages
+	 *
+	 * Note that mnt_context_get_excode() is used for backward compatibility and
+	 * will fill @buf with error messages from mnt_context_get_mesgs(). Therefore,
+	 * calling libmount_mesgs(cxt, 'e') is currently unnecessary.
+	 */
 	if (*buf) {
 		const char *spec = tgt;
 		if (!spec)
@@ -388,6 +432,14 @@ static int mk_exit_code(struct libmnt_context *cxt, int rc)
 			fprintf(stderr, _("       dmesg(1) may have more information after failed mount system call.\n"));
 	}
 
+	/* warning messages */
+	libmount_mesgs(cxt, 'w');
+
+	/* info messages */
+	if (mnt_context_is_verbose(cxt))
+		libmount_mesgs(cxt, 'i');
+
+	/* extra mount(8) messages */
 	if (rc == MNT_EX_SUCCESS && mnt_context_get_status(cxt) == 1) {
 		selinux_warning(cxt, tgt);
 	}
@@ -455,10 +507,10 @@ static void append_option(struct libmnt_context *cxt, const char *opt, const cha
 {
 	char *o = NULL;
 
-	if (opt && (*opt == '=' || *opt == '\'' || *opt == '\"' || isblank(*opt)))
+	if (opt && !ul_optstr_is_valid(opt))
 		errx(MNT_EX_USAGE, _("unsupported option format: %s"), opt);
 
-	if (arg && *arg)
+	if (opt && arg && *arg)
 		xasprintf(&o, "%s=\"%s\"", opt, arg);
 
 	if (mnt_context_append_options(cxt, o ? : opt))
@@ -482,7 +534,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	FILE *out = stdout;
 
 	fputs(USAGE_HEADER, out);
-	printf(_(
+	fprintf(out, _(
 		" %1$s [-lhV]\n"
 		" %1$s -a [options]\n"
 		" %1$s [options] [--source] <source> | [--target] <directory>\n"
@@ -501,6 +553,12 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -T, --fstab <path>      alternative file to /etc/fstab\n"), out);
 	fputs(_(" -i, --internal-only     don't call the mount.<type> helpers\n"), out);
 	fputs(_(" -l, --show-labels       show also filesystem labels\n"), out);
+	fputs(_("     --map-groups <inner>:<outer>:<count>\n"
+		"                         add the specified GID map to an ID-mapped mount\n"), out);
+	fputs(_("     --map-users <inner>:<outer>:<count>\n"
+		"                         add the specified UID map to an ID-mapped mount\n"), out);
+	fputs(_("     --map-users /proc/<pid>/ns/user\n"
+		"                         specify the user namespace for an ID-mapped mount\n"), out);
 	fputs(_(" -m, --mkdir[=<mode>]    alias to '-o X-mount.mkdir[=<mode>]'\n"), out);
 	fputs(_(" -n, --no-mtab           don't write to /etc/mtab\n"), out);
 	fputs(_("     --options-mode <mode>\n"
@@ -523,7 +581,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -N, --namespace <ns>    perform mount in another namespace\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(25));
+	fprintf(out, USAGE_HELP_OPTIONS(25));
 
 	fputs(USAGE_SEPARATOR, out);
 	fputs(_("Source:\n"), out);
@@ -552,7 +610,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" --make-rprivate         recursively mark a whole subtree as private\n"), out);
 	fputs(_(" --make-runbindable      recursively mark a whole subtree as unbindable\n"), out);
 
-	printf(USAGE_MAN_TAIL("mount(8)"));
+	fprintf(out, USAGE_MAN_TAIL("mount(8)"));
 
 	exit(MNT_EX_SUCCESS);
 }
@@ -615,6 +673,7 @@ int main(int argc, char **argv)
 	int c, rc = MNT_EX_SUCCESS, all = 0, show_labels = 0;
 	struct libmnt_context *cxt;
 	struct libmnt_table *fstab = NULL;
+	char *idmap = NULL;
 	char *srcbuf = NULL;
 	char *types = NULL;
 	int oper = 0, is_move = 0;
@@ -630,6 +689,8 @@ int main(int argc, char **argv)
 		MOUNT_OPT_RSLAVE,
 		MOUNT_OPT_RPRIVATE,
 		MOUNT_OPT_RUNBINDABLE,
+		MOUNT_OPT_MAP_GROUPS,
+		MOUNT_OPT_MAP_USERS,
 		MOUNT_OPT_TARGET,
 		MOUNT_OPT_TARGET_PREFIX,
 		MOUNT_OPT_SOURCE,
@@ -668,6 +729,8 @@ int main(int argc, char **argv)
 		{ "make-rslave",      no_argument,       NULL, MOUNT_OPT_RSLAVE      },
 		{ "make-rprivate",    no_argument,       NULL, MOUNT_OPT_RPRIVATE    },
 		{ "make-runbindable", no_argument,       NULL, MOUNT_OPT_RUNBINDABLE },
+		{ "map-groups",       required_argument, NULL, MOUNT_OPT_MAP_GROUPS  },
+		{ "map-users",        required_argument, NULL, MOUNT_OPT_MAP_USERS   },
 		{ "mkdir",            optional_argument, NULL, 'm'                   },
 		{ "no-canonicalize",  no_argument,       NULL, 'c'                   },
 		{ "internal-only",    no_argument,       NULL, 'i'                   },
@@ -850,6 +913,23 @@ int main(int argc, char **argv)
 			append_option(cxt, "runbindable", NULL);
 			propa = 1;
 			break;
+		case MOUNT_OPT_MAP_GROUPS:
+		case MOUNT_OPT_MAP_USERS:
+			if (*optarg == '=')
+				optarg++;
+			if (idmap && (*idmap == '/' || *optarg == '/')) {
+				warnx(_("bad usage"));
+				errtryhelp(MNT_EX_USAGE);
+			} else if (*optarg == '/') {
+				idmap = xstrdup(optarg);
+			} else {
+				char *tmp;
+				xasprintf(&tmp, "%s%s%s%s", idmap ? idmap : "", idmap ? " " : "",
+					c == MOUNT_OPT_MAP_GROUPS ? "g:" : "u:", optarg);
+				free(idmap);
+				idmap = tmp;
+			}
+			break;
 		case MOUNT_OPT_TARGET:
 			mnt_context_disable_swapmatch(cxt, 1);
 			mnt_context_set_target(cxt, optarg);
@@ -898,6 +978,9 @@ int main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
+	if (idmap)
+		append_option(cxt, "X-mount.idmap", idmap);
+
 	optmode |= optmode_mode | optmode_src;
 	if (optmode) {
 		if (!optmode_mode)
