@@ -61,6 +61,11 @@
 #include <sys/types.h>
 #include <grp.h>
 
+#if defined(USE_SYSTEMD) && HAVE_DECL_SD_SESSION_GET_USERNAME == 1
+# include <systemd/sd-login.h>
+# include <systemd/sd-daemon.h>
+#endif
+
 #include "nls.h"
 #include "xalloc.h"
 #include "strutils.h"
@@ -73,6 +78,7 @@
 #include "closestream.h"
 #include "timeutils.h"
 #include "pwdutils.h"
+#include "strv.h"
 
 #define	TERM_WIDTH	79
 #define	WRITE_TIME_OUT	300		/* in seconds */
@@ -96,8 +102,8 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -n, --nobanner          do not print banner, works only for root\n"), out);
 	fputs(_(" -t, --timeout <timeout> write timeout in seconds\n"), out);
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(25));
-	printf(USAGE_MAN_TAIL("wall(1)"));
+	fprintf(out, USAGE_HELP_OPTIONS(25));
+	fprintf(out, USAGE_MAN_TAIL("wall(1)"));
 
 	exit(EXIT_SUCCESS);
 }
@@ -131,10 +137,17 @@ static gid_t get_group_gid(const char *group)
 
 static struct group_workspace *init_group_workspace(const char *group)
 {
-	struct group_workspace *buf = xmalloc(sizeof(struct group_workspace));
+	struct group_workspace *buf;
+	long n;
+
+	n = sysconf(_SC_NGROUPS_MAX);
+	if (n < 0 || n > INT_MAX - 1)
+		return NULL;
+
+	buf = xmalloc(sizeof(struct group_workspace));
+	buf->ngroups = n + 1;	/* room for the primary gid */
 
 	buf->requested_group = get_group_gid(group);
-	buf->ngroups = sysconf(_SC_NGROUPS_MAX) + 1;  /* room for the primary gid */
 	buf->groups = xcalloc(buf->ngroups, sizeof(*buf->groups));
 
 	return buf;
@@ -178,19 +191,30 @@ static int is_gr_member(const char *login, const struct group_workspace *buf)
 	return 0;
 }
 
+static int has_tty(char **ttys, char *name)
+{
+	char **str;
+
+	UL_STRV_FOREACH(str, ttys) {
+		if (strcmp(*str, name) == 0)
+			return 1;
+	}
+
+	return 0;
+}
+
 int main(int argc, char **argv)
 {
 	int ch;
 	struct iovec iov;
 	struct utmpx *utmpptr;
-	char *p;
 	char line[sizeof(utmpptr->ut_line) + 1];
 	int print_banner = TRUE;
 	struct group_workspace *group_buf = NULL;
 	char *mbuf, *fname = NULL;
 	size_t mbufsize;
 	unsigned timeout = WRITE_TIME_OUT;
-	char **mvec = NULL;
+	char **mvec = NULL, **ttys = NULL, **str;
 	int mvecsz = 0;
 
 	static const struct option longopts[] = {
@@ -246,29 +270,73 @@ int main(int argc, char **argv)
 
 	iov.iov_base = mbuf;
 	iov.iov_len = mbufsize;
-	while((utmpptr = getutxent())) {
-		if (!utmpptr->ut_user[0])
-			continue;
-#ifdef USER_PROCESS
-		if (utmpptr->ut_type != USER_PROCESS)
-			continue;
-#endif
-		/* Joey Hess reports that use-sessreg in /etc/X11/wdm/ produces
-		 * ut_line entries like :0, and a write to /dev/:0 fails.
-		 *
-		 * It also seems that some login manager may produce empty ut_line.
-		 */
-		if (!*utmpptr->ut_line || *utmpptr->ut_line == ':')
-			continue;
 
-		if (group_buf && !is_gr_member(utmpptr->ut_user, group_buf))
-			continue;
+#if defined(USE_SYSTEMD) && HAVE_DECL_SD_SESSION_GET_USERNAME == 1
+	if (sd_booted() > 0) {
+		char **sessions_list;
+		int sessions;
 
-		mem2strcpy(line, utmpptr->ut_line, sizeof(utmpptr->ut_line), sizeof(line));
-		if ((p = ttymsg(&iov, 1, line, timeout)) != NULL)
-			warnx("%s", p);
+		sessions = sd_get_sessions(&sessions_list);
+		if (sessions < 0) {
+			warnx(_("error getting sessions: %s"), strerror(-sessions));
+			goto utmp;
+		}
+
+		for (int i = 0; i < sessions; i++) {
+			char *name, *tty;
+			int r;
+
+			if ((r = sd_session_get_username(sessions_list[i], &name)) < 0) {
+				warnx(_("get user name failed: %s"), strerror (-r));
+				goto utmp;
+			}
+			if (!(group_buf && !is_gr_member(name, group_buf))
+			    && sd_session_get_tty(sessions_list[i], &tty) >= 0
+			    && ul_strv_consume(&ttys, tty) < 0)
+				err(EXIT_FAILURE, _("failed to allocate lines list"));
+
+			free(name);
+			free(sessions_list[i]);
+		}
+		free(sessions_list);
 	}
-	endutxent();
+utmp:
+#endif
+	{
+		while ((utmpptr = getutxent())) {
+			if (!utmpptr->ut_user[0])
+				continue;
+#ifdef USER_PROCESS
+			if (utmpptr->ut_type != USER_PROCESS)
+				continue;
+#endif
+			/* Joey Hess reports that use-sessreg in /etc/X11/wdm/ produces
+			 * ut_line entries like :0, and a write to /dev/:0 fails.
+			 *
+			 * It also seems that some login manager may produce empty ut_line.
+			 */
+			if (!*utmpptr->ut_line || *utmpptr->ut_line == ':')
+				continue;
+
+			if (group_buf && !is_gr_member(utmpptr->ut_user, group_buf))
+				continue;
+
+			mem2strcpy(line, utmpptr->ut_line, sizeof(utmpptr->ut_line), sizeof(line));
+			if (has_tty(ttys, line))
+				continue;
+			if (ul_strv_extend(&ttys, line) < 0)
+				err(EXIT_FAILURE, _("failed to allocate lines list"));
+		}
+		endutxent();
+	}
+
+	UL_STRV_FOREACH(str, ttys) {
+		char *er = ttymsg(&iov, 1, *str, timeout);
+		if (er)
+			 warnx("%s", er);
+	}
+
+	ul_strv_free(ttys);
 	free(mbuf);
 	free_group_workspace(group_buf);
 	exit(EXIT_SUCCESS);
@@ -284,10 +352,10 @@ static char *makemsg(char *fname, char **mvec, int mvecsz,
 
 	if (print_banner == TRUE) {
 		char *hostname = xgethostname();
-		char *whom, *where, date[CTIME_BUFSIZ];
+		char *whombuf, *whom, *where, date[CTIME_BUFSIZ];
 		time_t now;
 
-		whom = xgetlogin();
+		whombuf = whom = xgetlogin();
 		if (!whom) {
 			whom = "<someone>";
 			warn(_("cannot get passwd uid"));
@@ -318,6 +386,7 @@ static char *makemsg(char *fname, char **mvec, int mvecsz,
 				whom, hostname, where, date);
 		fprintf(fs, "%-*.*s\007\007\r\n", TERM_WIDTH, TERM_WIDTH, lbuf);
 		free(hostname);
+		free(whombuf);
 	}
 	fprintf(fs, "%*s\r\n", TERM_WIDTH, " ");
 
