@@ -1,6 +1,12 @@
-
+/*
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * Copyright (C) 2008 Karel Zak <kzak@redhat.com>
+ */
 #include <stdio.h>
 #include <unistd.h>
+#include <stdlib.h>
+#include <err.h>
 
 #include "md5.h"
 
@@ -19,7 +25,9 @@ int main(void)
 			ul_MD5Update( &ctx, buf, ret );
 	}
 
-	fclose(stdin);
+	if(freopen ("/dev/null", "r", stdin) == NULL)
+		err(EXIT_FAILURE, "stdin->null failed!");
+
 	ul_MD5Final( digest, &ctx );
 
 	for (i = 0; i < UL_MD5LENGTH; i++)
