@@ -48,6 +48,7 @@
 #include "c.h"
 #include "widechar.h"
 #include "closestream.h"
+#include "fgetwc_or_err.h"
 
 /*
 COLRM removes unwanted columns from a file
@@ -62,13 +63,11 @@ static void __attribute__((__noreturn__)) usage(void)
 		       program_invocation_short_name);
 
 	fputs(USAGE_SEPARATOR, out);
-	fputs(_("Filter out the specified columns.\n"), out);
+	fputs(_("Filter out the specified columns from standard input.\n"), out);
 
 	fputs(USAGE_OPTIONS, out);
-	printf(USAGE_HELP_OPTIONS(16));
-	fprintf(out, _("%s reads from standard input and writes to standard output\n\n"),
-		       program_invocation_short_name);
-	printf(USAGE_MAN_TAIL("colrm(1)"));
+	fprintf(out, USAGE_HELP_OPTIONS(16));
+	fprintf(out, USAGE_MAN_TAIL("colrm(1)"));
 	exit(EXIT_SUCCESS);
 }
 
@@ -81,7 +80,7 @@ static int process_input(unsigned long first, unsigned long last)
 	int padding;
 
 	for (;;) {
-		c = getwc(stdin);
+		c = fgetwc_or_err(stdin);
 		if (c == WEOF)
 			return 0;
 		if (c == '\t')
@@ -112,7 +111,7 @@ static int process_input(unsigned long first, unsigned long last)
 
 	/* Loop getting rid of characters */
 	while (!last || ct < last) {
-		c = getwc(stdin);
+		c = fgetwc_or_err(stdin);
 		if (c == WEOF)
 			return 0;
 		if (c == '\n') {
@@ -135,7 +134,7 @@ static int process_input(unsigned long first, unsigned long last)
 
 	/* Output last of the line */
 	for (;;) {
-		c = getwc(stdin);
+		c = fgetwc_or_err(stdin);
 		if (c == WEOF)
 			break;
 		if (c == '\n') {
